/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.api.cloud.param;

import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

/**
 * 云端添加监控异常
 *
 * @author xiang
 * @date  2023/09/18 11:09
 **/
@Getter
@Setter
public class DepotCloudAlarmAddParam {

    /** 项目编码 */
    @ApiModelProperty(value = "项目编码", position = 2)
    private String depotId;

    /** 进程分类
     * 磅房 入库 车辆离开门禁区到达扦样区校验超时时间       预警类型  1
     * 磅房 入库 车辆离开扦样区到达称重区进行一次过磅称重   预警类型  2
     * 磅房 入库 车辆离开库房装卸区到达称重区进行二次过磅称重   预警类型  3
     * 磅房 入库 系统校验皮重。净重或者校验是否存在一致重量(上下阈值)  预警类型  4 弃用
     * 磅房 入库 车辆皮重超出系统配置的皮重阈值,请管理员及时处理预警信息  预警类型  41
     * 磅房 入库 车辆净重超出系统配置的净重重阈值,请管理员及时处理预警信息  预警类型  42
     * 磅房 入库 系统中存在当前净重上下阈值范围内记录,请管理员及时处理预警信息  预警类型  43
     * 磅房 出库 车辆进行一次过磅称重   预警类型  5
     * 磅房 出入库 二次过磅到门禁超时   预警类型  81/82
     *
     * 项目心跳监控  6
     * 设备离线  7
     * 车辆身份识别报警  8
     *
     * 云端 入库 完成一次过磅到达卸货区拍照上传校验超时   预警类型  21
     * 云端 入库 扦样价格过高   预警类型  22
     * 云端 入库 扦样水份过高   预警类型  23
     * 云端 出库 关单报出场超时   预警类型  24
     */
    @ApiModelProperty(value = "进程分类", position = 6)
    private String flowType;

    /** 异常信息 */
    @ApiModelProperty(value = "异常信息", position = 8)
    private String alarmMsg;

    /** 创建时间 */
    @ApiModelProperty(value = "创建时间", position = 8)
    private String createTime;

    /** ******************************************  出入库记录补充    ****************************************** */

    /** 预约ID */
    @ApiModelProperty(value = "预约ID", position = 3)
    private String truckBookId;
    /** ******************************************  车辆身份识别    ****************************************** */
    //车牌
    private String licensePlate;
    /** 出入库:入库/出库 */
    private String depotBizType;
    //区块分类:入口摄像头/扦样摄像头/地磅摄像头/出口摄像头
    private String areaType;

    /** ******************************************  设备记录补充    ****************************************** */

    /** 设备名称 */
    @ApiModelProperty(value = "设备名称", position = 6)
    private String deviceName;

    /** 设备编号 */
    @ApiModelProperty(value = "设备编号", position = 5)
    private String deviceTax;

    public DepotCloudAlarmAddParam() {
    }

    public DepotCloudAlarmAddParam(String truckBookId, String flowType, String alarmMsg) {
        this.flowType = flowType;
        this.alarmMsg = alarmMsg;
        this.truckBookId = truckBookId;
    }
}
