/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizarea.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.bizarea.entity.BizArea;
import vip.xiaonuo.biz.modular.bizarea.param.BizAreaAddParam;
import vip.xiaonuo.biz.modular.bizarea.param.BizAreaEditParam;
import vip.xiaonuo.biz.modular.bizarea.param.BizAreaIdParam;
import vip.xiaonuo.biz.modular.bizarea.param.BizAreaPageParam;
import vip.xiaonuo.biz.modular.bizarea.service.BizAreaService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 区域信息控制器
 *
 * @author fanzherong
 * @date  2025/02/17 08:54
 */
@Api(tags = "区域信息控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class BizAreaController {

    @Resource
    private BizAreaService bizAreaService;

    /**
     * 获取区域信息分页
     *
     * @author fanzherong
     * @date  2025/02/17 08:54
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取区域信息分页")
    @SaCheckPermission("/biz/bizarea/page")
    @GetMapping("/biz/bizarea/page")
    public CommonResult<Page<BizArea>> page(BizAreaPageParam bizAreaPageParam) {
        return CommonResult.data(bizAreaService.page(bizAreaPageParam));
    }

    /**
     * 添加区域信息
     *
     * @author fanzherong
     * @date  2025/02/17 08:54
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加区域信息")
    @CommonLog("添加区域信息")
    @SaCheckPermission("/biz/bizarea/add")
    @PostMapping("/biz/bizarea/add")
    public CommonResult<String> add(@RequestBody @Valid BizAreaAddParam bizAreaAddParam) {
        bizAreaService.add(bizAreaAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑区域信息
     *
     * @author fanzherong
     * @date  2025/02/17 08:54
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑区域信息")
    @CommonLog("编辑区域信息")
    @SaCheckPermission("/biz/bizarea/edit")
    @PostMapping("/biz/bizarea/edit")
    public CommonResult<String> edit(@RequestBody @Valid BizAreaEditParam bizAreaEditParam) {
        bizAreaService.edit(bizAreaEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除区域信息
     *
     * @author fanzherong
     * @date  2025/02/17 08:54
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除区域信息")
    @CommonLog("删除区域信息")
    @SaCheckPermission("/biz/bizarea/delete")
    @PostMapping("/biz/bizarea/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<BizAreaIdParam> bizAreaIdParamList) {
        bizAreaService.delete(bizAreaIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取区域信息详情
     *
     * @author fanzherong
     * @date  2025/02/17 08:54
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取区域信息详情")
    @SaCheckPermission("/biz/bizarea/detail")
    @GetMapping("/biz/bizarea/detail")
    public CommonResult<BizArea> detail(@Valid BizAreaIdParam bizAreaIdParam) {
        return CommonResult.data(bizAreaService.detail(bizAreaIdParam));
    }

    @ApiOperationSupport(order = 6)
    @ApiOperation("获取区域信息列表")
    @SaCheckPermission("/biz/bizarea/areaList")
    @GetMapping("/biz/bizarea/areaList")
    public CommonResult<List<BizArea>> areaList() {
        return CommonResult.data(bizAreaService.areaList());
    }

    @ApiOperationSupport(order = 6)
    @ApiOperation("小程序获取区域信息列表")
    @SaCheckPermission("/api/mini/biz/bizarea/areaList")
    @GetMapping("/api/mini/biz/bizarea/areaList")
    public CommonResult<List<BizArea>> areaMiniList() {
        return CommonResult.data(bizAreaService.areaListByUser());
    }
}
