/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizarea.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.auth.core.util.StpLoginUserUtil;
import vip.xiaonuo.biz.modular.bizvideo.entity.BizVideo;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.biz.modular.bizarea.entity.BizArea;
import vip.xiaonuo.biz.modular.bizarea.mapper.BizAreaMapper;
import vip.xiaonuo.biz.modular.bizarea.param.BizAreaAddParam;
import vip.xiaonuo.biz.modular.bizarea.param.BizAreaEditParam;
import vip.xiaonuo.biz.modular.bizarea.param.BizAreaIdParam;
import vip.xiaonuo.biz.modular.bizarea.param.BizAreaPageParam;
import vip.xiaonuo.biz.modular.bizarea.service.BizAreaService;

import java.util.ArrayList;
import java.util.List;

/**
 * 区域信息Service接口实现类
 *
 * @author fanzherong
 * @date  2025/02/17 08:54
 **/
@Service
public class BizAreaServiceImpl extends ServiceImpl<BizAreaMapper, BizArea> implements BizAreaService {

    @Override
    public Page<BizArea> page(BizAreaPageParam bizAreaPageParam) {
        QueryWrapper<BizArea> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(bizAreaPageParam.getAreaName())){
            queryWrapper.lambda().like(BizArea::getAreaName,bizAreaPageParam.getAreaName());
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Override
    public void add(BizAreaAddParam bizAreaAddParam) {
        checkParam(bizAreaAddParam.getAreaName());
        BizArea bizArea = BeanUtil.toBean(bizAreaAddParam, BizArea.class);
        this.save(bizArea);
    }

    public void checkParam(String areaName){
        long count = this.count(new QueryWrapper<BizArea>().lambda().eq(BizArea::getAreaName, areaName));
        if(count>0){
            throw new CommonException("当前区域名称已经存在！");
        }
    }

    @Override
    public void edit(BizAreaEditParam bizAreaEditParam) {
        BizArea bizArea = this.queryEntity(bizAreaEditParam.getId());
        if(!StringUtils.equals(bizAreaEditParam.getAreaName(),bizArea.getAreaName())){
            checkParam(bizArea.getAreaName());
        }
        BeanUtil.copyProperties(bizAreaEditParam, bizArea);
        this.updateById(bizArea);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<BizAreaIdParam> bizAreaIdParamList) {
        // 执行删除
        this.removeBatchByIds(CollStreamUtil.toList(bizAreaIdParamList, BizAreaIdParam::getId));
    }

    @Override
    public BizArea detail(BizAreaIdParam bizAreaIdParam) {
        return this.queryEntity(bizAreaIdParam.getId());
    }

    @Override
    public BizArea queryEntity(String id) {
        BizArea bizArea = this.getById(id);
        if(ObjectUtil.isEmpty(bizArea)) {
            throw new CommonException("区域信息不存在，id值为：{}", id);
        }
        return bizArea;
    }

    @Override
    public List<BizArea> areaList() {
        List<BizArea> list = this.list();
        return list;
    }

    @Override
    public List<BizArea> areaListByUser() {
        QueryWrapper<BizArea> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("buv.delete_flag","NOT_DELETE");
        queryWrapper.eq("ba.delete_flag","NOT_DELETE");
        // 判断当前是否视频监察员角色
        if(StpLoginUserUtil.getLoginUser().getRoleCodeList().contains("54ahw3iob9")){
            queryWrapper.eq("buv.user_id",StpLoginUserUtil.getLoginUser().getId());
        }else{
            return new ArrayList<>();
        }
        return this.getBaseMapper().areaListByUser(queryWrapper);
    }
}
