/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotAlarmLog.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.biz.modular.depotGrain.service.DepotGrainService;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.biz.modular.depotAlarmLog.entity.DepotAlarmLog;
import vip.xiaonuo.biz.modular.depotAlarmLog.mapper.DepotAlarmLogMapper;
import vip.xiaonuo.biz.modular.depotAlarmLog.param.DepotAlarmLogAddParam;
import vip.xiaonuo.biz.modular.depotAlarmLog.param.DepotAlarmLogEditParam;
import vip.xiaonuo.biz.modular.depotAlarmLog.param.DepotAlarmLogIdParam;
import vip.xiaonuo.biz.modular.depotAlarmLog.param.DepotAlarmLogPageParam;

import javax.annotation.Resource;
import java.util.List;

/**
 * 异常和报警处理日志Service接口实现类
 *
 * @author xiang
 * @date 2023/09/19 17:08
 **/
@Service
public class DepotAlarmLogService extends ServiceImpl<DepotAlarmLogMapper, DepotAlarmLog> {
    @Resource
    DepotGrainService depotGrainService;

    public Page<DepotAlarmLog> page(DepotAlarmLogPageParam depotAlarmLogPageParam) {
        QueryWrapper<DepotAlarmLog> queryWrapper = new QueryWrapper<>();
        List<String> depotIdLst = depotGrainService.lstOrgGrep();
        queryWrapper.lambda().in(CollUtil.isNotEmpty(depotIdLst), DepotAlarmLog::getDepotId, depotIdLst);
        if (ObjectUtil.isNotEmpty(depotAlarmLogPageParam.getDepotId())) {
            queryWrapper.lambda().eq(DepotAlarmLog::getDepotId, depotAlarmLogPageParam.getDepotId());
        }
        if (ObjectUtil.isNotEmpty(depotAlarmLogPageParam.getLogMsg())) {
            queryWrapper.lambda().like(DepotAlarmLog::getLogMsg, depotAlarmLogPageParam.getLogMsg());
        }
        if (ObjectUtil.isAllNotEmpty(depotAlarmLogPageParam.getSortField(), depotAlarmLogPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(depotAlarmLogPageParam.getSortOrder());
            queryWrapper.orderBy(true, depotAlarmLogPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(depotAlarmLogPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByDesc(DepotAlarmLog::getId);
        }
        Page page = this.page(CommonPageRequest.defaultPage(), queryWrapper);
        //匹配粮库名称
        depotGrainService.putDepotName(page.getRecords());
        return page;
    }

    public void add(DepotAlarmLogAddParam depotAlarmLogAddParam) {
        DepotAlarmLog depotAlarmLog = BeanUtil.toBean(depotAlarmLogAddParam, DepotAlarmLog.class);
        this.save(depotAlarmLog);
    }

    public void edit(DepotAlarmLogEditParam depotAlarmLogEditParam) {
        DepotAlarmLog depotAlarmLog = this.queryEntity(depotAlarmLogEditParam.getId());
        BeanUtil.copyProperties(depotAlarmLogEditParam, depotAlarmLog);
        this.updateById(depotAlarmLog);
    }

    @Transactional(rollbackFor = Exception.class)
    public void delete(List<DepotAlarmLogIdParam> depotAlarmLogIdParamList) {
        // 执行删除
        this.removeBatchByIds(CollStreamUtil.toList(depotAlarmLogIdParamList, DepotAlarmLogIdParam::getId));
    }

    public DepotAlarmLog detail(DepotAlarmLogIdParam depotAlarmLogIdParam) {
        return this.queryEntity(depotAlarmLogIdParam.getId());
    }

    public DepotAlarmLog queryEntity(String id) {
        DepotAlarmLog depotAlarmLog = this.getById(id);
        if (ObjectUtil.isEmpty(depotAlarmLog)) {
            throw new CommonException("异常和报警处理日志不存在，id值为：{}", id);
        }
        return depotAlarmLog;
    }

}
