/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotCloudMonitor.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.biz.modular.depotGrain.service.DepotGrainService;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.biz.modular.depotCloudMonitor.entity.DepotCloudMonitor;
import vip.xiaonuo.biz.modular.depotCloudMonitor.mapper.DepotCloudMonitorMapper;
import vip.xiaonuo.biz.modular.depotCloudMonitor.param.DepotCloudMonitorAddParam;
import vip.xiaonuo.biz.modular.depotCloudMonitor.param.DepotCloudMonitorEditParam;
import vip.xiaonuo.biz.modular.depotCloudMonitor.param.DepotCloudMonitorIdParam;
import vip.xiaonuo.biz.modular.depotCloudMonitor.param.DepotCloudMonitorPageParam;

import javax.annotation.Resource;
import java.util.List;

/**
 * 项目云监控表Service接口实现类
 *
 * @author xiang
 * @date 2023/11/03 13:40
 **/
@Service
public class DepotCloudMonitorService extends ServiceImpl<DepotCloudMonitorMapper, DepotCloudMonitor> {

    @Resource
    DepotGrainService depotGrainService;

    public Page<DepotCloudMonitor> page(DepotCloudMonitorPageParam param) {
        QueryWrapper<DepotCloudMonitor> queryWrapper = new QueryWrapper<>();
        List<String> depotIdLst = depotGrainService.lstOrgGrep();
        queryWrapper.lambda().in(CollUtil.isNotEmpty(depotIdLst), DepotCloudMonitor::getDepotId, depotIdLst);
        if (ObjectUtil.isNotEmpty(param.getDepotId())) {
            queryWrapper.lambda().eq(DepotCloudMonitor::getDepotId, param.getDepotId());
        }
        if (ObjectUtil.isAllNotEmpty(param.getSortField(), param.getSortOrder())) {
            CommonSortOrderEnum.validate(param.getSortOrder());
            queryWrapper.orderBy(true, param.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(param.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(DepotCloudMonitor::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    public void add(DepotCloudMonitorAddParam depotCloudMonitorAddParam) {
        DepotCloudMonitor depotCloudMonitor = BeanUtil.toBean(depotCloudMonitorAddParam, DepotCloudMonitor.class);
        this.save(depotCloudMonitor);
    }

    public void edit(DepotCloudMonitorEditParam depotCloudMonitorEditParam) {
        DepotCloudMonitor depotCloudMonitor = this.queryEntity(depotCloudMonitorEditParam.getId());
        BeanUtil.copyProperties(depotCloudMonitorEditParam, depotCloudMonitor);
        this.updateById(depotCloudMonitor);
    }

    @Transactional(rollbackFor = Exception.class)
    public void delete(List<DepotCloudMonitorIdParam> depotCloudMonitorIdParamList) {
        // 执行删除
        this.removeBatchByIds(CollStreamUtil.toList(depotCloudMonitorIdParamList, DepotCloudMonitorIdParam::getId));
    }

    public DepotCloudMonitor detail(DepotCloudMonitorIdParam depotCloudMonitorIdParam) {
        return this.queryEntity(depotCloudMonitorIdParam.getId());
    }

    public DepotCloudMonitor queryEntity(String id) {
        DepotCloudMonitor depotCloudMonitor = this.getById(id);
        if (ObjectUtil.isEmpty(depotCloudMonitor)) {
            throw new CommonException("项目云监控表不存在，id值为：{}", id);
        }
        return depotCloudMonitor;
    }
}
