/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotDeviceWarn.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.modular.depotDeviceWarn.param.*;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotDeviceWarn.entity.DepotDeviceWarn;
import vip.xiaonuo.biz.modular.depotDeviceWarn.service.DepotDeviceWarnService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 离线报警控制器
 *
 * @author xiang
 * @date 2023/09/19 13:59
 */
@Api(tags = "离线报警控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotDeviceWarnController {

    @Resource
    private DepotDeviceWarnService depotDeviceWarnService;

    /**
     * 获取离线报警分页
     *
     * @author xiang
     * @date 2023/09/19 13:59
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取离线报警分页")
    @SaCheckPermission("/biz/depotDeviceWarn/page")
    @GetMapping("/biz/depotDeviceWarn/page")
    public CommonResult<Page<DepotDeviceWarn>> page(DepotDeviceWarnPageParam depotDeviceWarnPageParam) {
        return CommonResult.data(depotDeviceWarnService.page(depotDeviceWarnPageParam));
    }

    /**
     * 添加离线报警
     *
     * @author xiang
     * @date 2023/09/19 13:59
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加离线报警")
    @CommonLog("添加离线报警")
    @SaCheckPermission("/biz/depotDeviceWarn/add")
    @PostMapping("/biz/depotDeviceWarn/add")
    public CommonResult<String> add(@RequestBody @Valid DepotDeviceWarnAddParam depotDeviceWarnAddParam) {
        depotDeviceWarnService.add(depotDeviceWarnAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑离线报警
     *
     * @author xiang
     * @date 2023/09/19 13:59
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑离线报警")
    @CommonLog("编辑离线报警")
    @SaCheckPermission("/biz/depotDeviceWarn/edit")
    @PostMapping("/biz/depotDeviceWarn/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotDeviceWarnEditParam depotDeviceWarnEditParam) {
        depotDeviceWarnService.edit(depotDeviceWarnEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除离线报警
     *
     * @author xiang
     * @date 2023/09/19 13:59
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除离线报警")
    @CommonLog("删除离线报警")
    @SaCheckPermission("/biz/depotDeviceWarn/delete")
    @PostMapping("/biz/depotDeviceWarn/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotDeviceWarnIdParam> depotDeviceWarnIdParamList) {
        depotDeviceWarnService.delete(depotDeviceWarnIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取离线报警详情
     *
     * @author xiang
     * @date 2023/09/19 13:59
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取离线报警详情")
    @SaCheckPermission("/biz/depotDeviceWarn/detail")
    @GetMapping("/biz/depotDeviceWarn/detail")
    public CommonResult<DepotDeviceWarn> detail(@Valid DepotDeviceWarnIdParam depotDeviceWarnIdParam) {
        return CommonResult.data(depotDeviceWarnService.detail(depotDeviceWarnIdParam));
    }

    /**
     * 设备离线预警处理
     *
     * @author xiang
     * @date 2023/09/19 13:59
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取离线报警详情")
    @SaCheckPermission("/biz/depotDeviceWarn/edit")
    @PostMapping("/biz/depotDeviceWarn/processTo")
    public CommonResult<DepotDeviceWarn> processTo(@RequestBody DepotDeviceWarnProcessParam param) {
        depotDeviceWarnService.processTo(param);
        return CommonResult.ok();
    }

}
