/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotFlyerSiteLog.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.logging.log4j.util.Strings;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.core.Const.BizStr;
import vip.xiaonuo.biz.modular.depotFlyerSiteLog.param.*;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotFlyerSiteLog.entity.DepotFlyerSiteLog;
import vip.xiaonuo.biz.modular.depotFlyerSiteLog.service.DepotFlyerSiteLogService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 飞行监管控制器
 *
 * @author xiang
 * @date 2023/09/19 09:12
 */
@Api(tags = "飞行监管控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotFlyerSiteLogController {

    @Resource
    private DepotFlyerSiteLogService depotFlyerSiteLogService;

    /**
     * 获取飞行监管分页
     *
     * @author xiang
     * @date 2023/09/19 09:12
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取飞行监管分页")
    @SaCheckPermission("/biz/depotFlyerSiteLog/page")
    @GetMapping("/biz/depotFlyerSiteLog/page")
    public CommonResult<Page<DepotFlyerSiteLog>> page(DepotFlyerSiteLogPageParam depotFlyerSiteLogPageParam) {
        return CommonResult.data(depotFlyerSiteLogService.page(depotFlyerSiteLogPageParam));
    }

    /**
     * 添加飞行监管
     *
     * @author xiang
     * @date 2023/09/19 09:12
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加飞行监管")
    @CommonLog("添加飞行监管")
    @SaCheckPermission("/biz/depotFlyerSiteLog/add")
    @PostMapping("/biz/depotFlyerSiteLog/add")
    public CommonResult<String> add(@RequestBody @Valid DepotFlyerSiteLogAddParam depotFlyerSiteLogAddParam) {
        depotFlyerSiteLogService.add(depotFlyerSiteLogAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑飞行监管
     *
     * @author xiang
     * @date 2023/09/19 09:12
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑飞行监管")
    @CommonLog("编辑飞行监管")
    @SaCheckPermission("/biz/depotFlyerSiteLog/edit")
    @PostMapping("/biz/depotFlyerSiteLog/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotFlyerSiteLogEditParam depotFlyerSiteLogEditParam) {
        depotFlyerSiteLogService.edit(depotFlyerSiteLogEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除飞行监管
     *
     * @author xiang
     * @date 2023/09/19 09:12
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除飞行监管")
    @CommonLog("删除飞行监管")
    @SaCheckPermission("/biz/depotFlyerSiteLog/delete")
    @PostMapping("/biz/depotFlyerSiteLog/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotFlyerSiteLogIdParam> depotFlyerSiteLogIdParamList) {
        depotFlyerSiteLogService.delete(depotFlyerSiteLogIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取飞行监管详情
     *
     * @author xiang
     * @date 2023/09/19 09:12
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取飞行监管详情")
    @SaCheckPermission("/biz/depotFlyerSiteLog/detail")
    @GetMapping("/biz/depotFlyerSiteLog/detail")
    public CommonResult<DepotFlyerSiteLog> detail(@Valid DepotFlyerSiteLogIdParam depotFlyerSiteLogIdParam) {
        return CommonResult.data(depotFlyerSiteLogService.detail(depotFlyerSiteLogIdParam));
    }

    /**
     * 编辑飞行监管
     *
     * @author xiang
     * @date 2023/09/19 09:12
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑飞行监管")
    @CommonLog("编辑飞行监管")
    @SaCheckPermission("/biz/depotFlyerSiteLog/edit")
    @PostMapping("/biz/depotFlyerSiteLog/processFinish")
    public CommonResult<String> processFinish(@RequestBody @Valid DepotFlyerSiteLogEditParam param) {
        if (Strings.isBlank(param.getSaveState()))
            param.setSaveState(BizStr.STATE_PROCESS_E);
        depotFlyerSiteLogService.processFinish(param);
        return CommonResult.ok();
    }
}
