/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotGrainInfo.entity;

import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import vip.xiaonuo.biz.modular.depotGrainInfoPrice.entity.BaseDepotGrainInfoPrice;

import java.math.BigDecimal;

/**
 * 企业客户信息表实体
 *
 * @author xiang
 * @date  2023/09/14 15:00
 **/
@Getter
@Setter
@TableName("DEPOT_GRAIN_INFO")
public class DepotGrainInfo extends BaseDepotGrainInfoPrice {

    /** ID */
    @TableId
    @ApiModelProperty(value = "ID", position = 1)
    private String id;

    /** 项目编码 */
    @ApiModelProperty(value = "项目编码", position = 2)
    private String depotId;

    /** 贷款合同号 */
    @ApiModelProperty(value = "贷款合同号", position = 3)
    private String contractCode;

    /** 位置(市) */
    @ApiModelProperty(value = "位置(市)", position = 4)
    private String addressCity;

    /** 位置(县) */
    @ApiModelProperty(value = "位置(县)", position = 5)
    private String addressCounty;

    /** 位置(镇) */
    @ApiModelProperty(value = "位置(镇)", position = 6)
    private String addressTown;

    /** 库容(万吨) */
    @ApiModelProperty(value = "库容(万吨)", position = 7)
    private String depotSize;

    /** 日烘干能力(吨) */
    @ApiModelProperty(value = "日烘干能力(吨)", position = 8)
    private String dayDrySize;

    /** 当前库存数量(万吨) */
    @ApiModelProperty(value = "当前库存数量(万吨)", position = 9)
    private BigDecimal depotCount;

    /** 贷款额度 */
    @ApiModelProperty(value = "贷款额度", position = 10)
    private BigDecimal loanLimit;

    /** 贷款金额 */
    @ApiModelProperty(value = "贷款金额", position = 11)
    private BigDecimal loanSum;

    /** 贷款余额 */
    @ApiModelProperty(value = "贷款余额", position = 12)
    private BigDecimal loanBalance;

    /** 贷款利息 */
    @ApiModelProperty(value = "贷款利息", position = 13)
    private BigDecimal loanInterest;

    /** 贷款到期日 */
    @ApiModelProperty(value = "贷款到期日", position = 14)
    private String loanEndDate;

    /** 纳税识别号 */
    @ApiModelProperty(value = "纳税识别号", position = 15)
    private String creditCode;

    /** 已质押库存数(万吨) */
    @ApiModelProperty(value = "已质押库存数(万吨)", position = 16)
    private BigDecimal mortgageCount;

    /** 入库待申请数量(吨) */
    @ApiModelProperty(value = "入库待申请数量(吨)", position = 16)
    private BigDecimal tonInCount;

    /** 入库待申请车数(辆) */
    @ApiModelProperty(value = "入库待申请车数(辆)", position = 16)
    private BigDecimal truckInCount;

    /** 入库待审批数量(吨) */
    @ApiModelProperty(value = "入库待审批数量(吨)", position = 16)
    private BigDecimal tonInCheckCount;

    /** 出库已审批数量(吨) */
    @ApiModelProperty(value = "出库已审批数量(吨)", position = 16)
    private BigDecimal tonOutFinishCount;

    /** 累计贷款金额 */
    @ApiModelProperty(value = "累计贷款金额", position = 21)
    private BigDecimal loanSumTotal;

    /** 累计还款金额 */
    @ApiModelProperty(value = "累计还款金额", position = 22)
    private BigDecimal repaySumTotal;

    /** 累计质押库存数(万吨) */
    @ApiModelProperty(value = "累计质押库存数(万吨)", position = 23)
    private BigDecimal mortgageCountTotal;

    /** 入库交易额 */
    @ApiModelProperty(value = "入库交易额", position = 24)
    private BigDecimal inAmount;

    /** 出库交易额 */
    @ApiModelProperty(value = "出库交易额", position = 25)
    private BigDecimal outAmount;

    public DepotGrainInfo() {
    }

    public DepotGrainInfo(String depotId) {
        this.depotId = depotId;
    }

    public DepotGrainInfo(String id, String depotId, String contractCode) {
        this.id = id;
        this.depotId = depotId;
        this.contractCode = contractCode;
    }
}
