/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotGrainInfoWarn.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotGrainInfoWarn.entity.DepotGrainInfoWarn;
import vip.xiaonuo.biz.modular.depotGrainInfoWarn.param.DepotGrainInfoWarnAddParam;
import vip.xiaonuo.biz.modular.depotGrainInfoWarn.param.DepotGrainInfoWarnEditParam;
import vip.xiaonuo.biz.modular.depotGrainInfoWarn.param.DepotGrainInfoWarnIdParam;
import vip.xiaonuo.biz.modular.depotGrainInfoWarn.param.DepotGrainInfoWarnPageParam;
import vip.xiaonuo.biz.modular.depotGrainInfoWarn.service.DepotGrainInfoWarnService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 客户预警处理控制器
 *
 * @author xiang
 * @date  2023/09/14 16:20
 */
@Api(tags = "客户预警处理控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotGrainInfoWarnController {

    @Resource
    private DepotGrainInfoWarnService depotGrainInfoWarnService;

    /**
     * 获取客户预警处理分页
     *
     * @author xiang
     * @date  2023/09/14 16:20
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取客户预警处理分页")
    @SaCheckPermission("/biz/depotGrainInfoWarn/page")
    @GetMapping("/biz/depotGrainInfoWarn/page")
    public CommonResult<Page<DepotGrainInfoWarn>> page(DepotGrainInfoWarnPageParam depotGrainInfoWarnPageParam) {
        return CommonResult.data(depotGrainInfoWarnService.page(depotGrainInfoWarnPageParam));
    }

    /**
     * 添加客户预警处理
     *
     * @author xiang
     * @date  2023/09/14 16:20
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加客户预警处理")
    @CommonLog("添加客户预警处理")
    @SaCheckPermission("/biz/depotGrainInfoWarn/add")
    @PostMapping("/biz/depotGrainInfoWarn/add")
    public CommonResult<String> add(@RequestBody @Valid DepotGrainInfoWarnAddParam depotGrainInfoWarnAddParam) {
        depotGrainInfoWarnService.add(depotGrainInfoWarnAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑客户预警处理
     *
     * @author xiang
     * @date  2023/09/14 16:20
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑客户预警处理")
    @CommonLog("编辑客户预警处理")
    @SaCheckPermission("/biz/depotGrainInfoWarn/edit")
    @PostMapping("/biz/depotGrainInfoWarn/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotGrainInfoWarnEditParam depotGrainInfoWarnEditParam) {
        depotGrainInfoWarnService.edit(depotGrainInfoWarnEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除客户预警处理
     *
     * @author xiang
     * @date  2023/09/14 16:20
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除客户预警处理")
    @CommonLog("删除客户预警处理")
    @SaCheckPermission("/biz/depotGrainInfoWarn/delete")
    @PostMapping("/biz/depotGrainInfoWarn/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<DepotGrainInfoWarnIdParam> depotGrainInfoWarnIdParamList) {
        depotGrainInfoWarnService.delete(depotGrainInfoWarnIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取客户预警处理详情
     *
     * @author xiang
     * @date  2023/09/14 16:20
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取客户预警处理详情")
    @SaCheckPermission("/biz/depotGrainInfoWarn/detail")
    @GetMapping("/biz/depotGrainInfoWarn/detail")
    public CommonResult<DepotGrainInfoWarn> detail(@Valid DepotGrainInfoWarnIdParam depotGrainInfoWarnIdParam) {
        return CommonResult.data(depotGrainInfoWarnService.detail(depotGrainInfoWarnIdParam));
    }
}
