/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotGrainShed.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.biz.core.exception.BizExceptionEnum;
import vip.xiaonuo.biz.modular.depotGrain.service.DepotGrainService;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.biz.modular.depotGrainShed.entity.DepotGrainShed;
import vip.xiaonuo.biz.modular.depotGrainShed.mapper.DepotGrainShedMapper;
import vip.xiaonuo.biz.modular.depotGrainShed.param.DepotGrainShedAddParam;
import vip.xiaonuo.biz.modular.depotGrainShed.param.DepotGrainShedEditParam;
import vip.xiaonuo.biz.modular.depotGrainShed.param.DepotGrainShedIdParam;
import vip.xiaonuo.biz.modular.depotGrainShed.param.DepotGrainShedPageParam;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

/**
 * 库位管理Service接口实现类
 *
 * @author xiang
 * @date 2023/09/14 09:15
 **/
@Service
public class DepotGrainShedService extends ServiceImpl<DepotGrainShedMapper, DepotGrainShed> {

    @Resource
    DepotGrainService depotGrainService;

    public Page<DepotGrainShed> page(DepotGrainShedPageParam param) {
        QueryWrapper<DepotGrainShed> wrapper = new QueryWrapper<>();
        List<String> depotIdLst = depotGrainService.lstOrgGrep();
        wrapper.lambda().in(CollUtil.isNotEmpty(depotIdLst), DepotGrainShed::getDepotId, depotIdLst);
        if (ObjectUtil.isNotEmpty(param.getDepotId())) {
            wrapper.lambda().eq(DepotGrainShed::getDepotId, param.getDepotId());
        }
        if (ObjectUtil.isNotEmpty(param.getShedName())) {
            wrapper.lambda().like(DepotGrainShed::getShedName, param.getShedName());
        }
        if (ObjectUtil.isNotEmpty(param.getRemark())) {
            wrapper.lambda().like(DepotGrainShed::getRemark, param.getRemark());
        }
        if (ObjectUtil.isAllNotEmpty(param.getSortField(), param.getSortOrder())) {
            CommonSortOrderEnum.validate(param.getSortOrder());
            wrapper.orderBy(true, param.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(param.getSortField()));
        } else {
            wrapper.lambda().orderByDesc(DepotGrainShed::getId);
        }
        Page page = this.page(CommonPageRequest.defaultPage(), wrapper);
        //匹配粮库名称
        depotGrainService.putDepotName(page.getRecords());
        return page;
    }

    public List<DepotGrainShed> lst(DepotGrainShedPageParam param) {
        LambdaQueryWrapper<DepotGrainShed> wrapper = new LambdaQueryWrapper<>(BeanUtil.toBean(param, DepotGrainShed.class));
        return this.list(wrapper);
    }

    public void add(DepotGrainShedAddParam addParam) {
        repeatName(null, addParam.getDepotId(), addParam.getShedName(), true);
        DepotGrainShed depotGrainShed = BeanUtil.toBean(addParam, DepotGrainShed.class);
        this.save(depotGrainShed);
    }

    public void edit(DepotGrainShedEditParam editParam) {
        repeatName(editParam.getId(), editParam.getDepotId(), editParam.getShedName(), true);
        DepotGrainShed depotGrainShed = this.queryEntity(editParam.getId());
        BeanUtil.copyProperties(editParam, depotGrainShed);
        this.updateById(depotGrainShed);
    }

    @Transactional(rollbackFor = Exception.class)
    public void delete(List<DepotGrainShedIdParam> depotGrainShedIdParamList) {
        // 执行删除
        this.removeBatchByIds(CollStreamUtil.toList(depotGrainShedIdParamList, DepotGrainShedIdParam::getId));
    }

    public DepotGrainShed detail(DepotGrainShedIdParam depotGrainShedIdParam) {
        return this.queryEntity(depotGrainShedIdParam.getId());
    }

    public DepotGrainShed queryEntity(String id) {
        DepotGrainShed depotGrainShed = this.getById(id);
        if (ObjectUtil.isEmpty(depotGrainShed)) {
            throw new CommonException("库位管理不存在，id值为：{}", id);
        }
        return depotGrainShed;
    }

    /**
     * 检查名称重复
     *
     * @param throwFlag true:抛异常
     * @return true:重复
     * @author xiang
     * @date 2023-01-01
     */
    private boolean repeatName(String id, String depotId, String name, boolean throwFlag) {
        List<DepotGrainShed> list = lst(new DepotGrainShedPageParam(depotId, name));
        if (list.isEmpty()) return false;
        if (Objects.equals(list.get(0).getId(), id) && list.size() == 1) return false;
        if (!throwFlag) return true;
        throw new CommonException(BizExceptionEnum.ERROR00005.getMessage());
    }
}
