/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotLoanGrant.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.core.Const.BizStr;
import vip.xiaonuo.biz.modular.depotLoanGrant.param.*;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotLoanGrant.entity.DepotLoanGrant;
import vip.xiaonuo.biz.modular.depotLoanGrant.service.DepotLoanGrantService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 入库放款单控制器
 *
 * @author xiang
 * @date 2023/09/18 16:03
 */
@Api(tags = "入库放款单控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotLoanGrantController {

    @Resource
    private DepotLoanGrantService depotLoanGrantService;

    /**
     * 获取入库放款单分页
     *
     * @author xiang
     * @date 2023/09/18 16:03
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取入库放款单分页")
    @SaCheckPermission("/biz/depotLoanGrant/page")
    @GetMapping("/biz/depotLoanGrant/page")
    public CommonResult<Page<DepotLoanGrant>> page(DepotLoanGrantPageParam depotLoanGrantPageParam) {
        return CommonResult.data(depotLoanGrantService.page(depotLoanGrantPageParam));
    }

    /**
     * 添加入库放款单
     *
     * @author xiang
     * @date 2023/09/18 16:03
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加入库放款单")
    @CommonLog("添加入库放款单")
    @SaCheckPermission("/biz/depotLoanGrant/add")
    @PostMapping("/biz/depotLoanGrant/add")
    public CommonResult<String> add(@RequestBody @Valid DepotLoanGrantAddParam depotLoanGrantAddParam) {
        depotLoanGrantService.add(depotLoanGrantAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑入库放款单
     *
     * @author xiang
     * @date 2023/09/18 16:03
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑入库放款单")
    @CommonLog("编辑入库放款单")
    @SaCheckPermission("/biz/depotLoanGrant/edit")
    @PostMapping("/biz/depotLoanGrant/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotLoanGrantEditParam depotLoanGrantEditParam) {
        depotLoanGrantService.edit(depotLoanGrantEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除入库放款单
     *
     * @author xiang
     * @date 2023/09/18 16:03
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除入库放款单")
    @CommonLog("删除入库放款单")
    @SaCheckPermission("/biz/depotLoanGrant/delete")
    @PostMapping("/biz/depotLoanGrant/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotLoanGrantIdParam> depotLoanGrantIdParamList) {
        depotLoanGrantService.delete(depotLoanGrantIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取入库放款单详情
     *
     * @author xiang
     * @date 2023/09/18 16:03
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取入库放款单详情")
    @SaCheckPermission("/biz/depotLoanGrant/detail")
    @GetMapping("/biz/depotLoanGrant/detail")
    public CommonResult<DepotLoanGrant> detail(@Valid DepotLoanGrantIdParam idParam) {
        return CommonResult.data("IdWorker".equals(idParam.getId()) ? new DepotLoanGrant(IdWorker.getIdStr(), BizStr.STATE_APPROVAL_N) : depotLoanGrantService.detail(idParam));
    }

    /**
     * 放款单二级审批
     *
     * @author xiang
     * @date 2023/09/14 09:44
     */
    @ApiOperationSupport(order = 6)
    @ApiOperation("放款单二级审批")
    @CommonLog("放款单二级审批")
    @SaCheckPermission("/biz/depotLoanGrant/bankOpApproval")
    @PostMapping("/biz/depotLoanGrant/bankOpApproval")
    public CommonResult<String> bankOpApproval(@RequestBody @Valid BankOpApprovalParam param) {
        depotLoanGrantService.bankOpApproval(param);
        return CommonResult.ok();
    }

}
