/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotLoanGrant.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import vip.xiaonuo.biz.modular.depotGrain.entity.BaseDepotGrainName;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 入库放款单实体
 *
 * @author xiang
 * @date  2023/09/18 16:03
 **/
@Getter
@Setter
@TableName("DEPOT_LOAN_GRANT")
public class DepotLoanGrant extends BaseDepotGrainName {

    /** ID */
    @TableId
    @ApiModelProperty(value = "ID", position = 1)
    private String id;

    /** 项目编码 */
    @ApiModelProperty(value = "项目编码", position = 2)
    private String depotId;

    /** 贷款合同编号 */
    @ApiModelProperty(value = "贷款合同编号", position = 3)
    private String contractCode;

    /** 放款单号 */
    @ApiModelProperty(value = "放款单号", position = 4)
    private String loanOrderNo;

    /** 粮食种类 */
    @ApiModelProperty(value = "粮食种类", position = 5)
    private String grainGroup;

    /** 入库价格（元/吨） */
    @ApiModelProperty(value = "入库价格（元/吨）", position = 6)
    private BigDecimal orderDryPrice;

    /** 入库实际价格（元/吨） */
    @ApiModelProperty(value = "入库实际价格（元/吨）", position = 6)
    private BigDecimal orderPrice;

    /** 入库重量（吨） */
    @ApiModelProperty(value = "入库重量（吨）", position = 7)
    private BigDecimal orderDryWeight;

    /** 入库实际重量（吨） */
    @ApiModelProperty(value = "入库实际重量（吨）", position = 7)
    private BigDecimal orderWeight;

    /** 入库总金额 */
    @ApiModelProperty(value = "入库总金额", position = 8)
    private BigDecimal orderAmount;

    /** 质押价格 */
    @ApiModelProperty(value = "质押价格", position = 9)
    private BigDecimal loanPrice;

    /** 平均质押价格 */
    @ApiModelProperty(value = "平均质押价格", position = 9)
    private BigDecimal loanPriceAvg;

    /** 质押率(%) */
    @ApiModelProperty(value = "质押率(%)", position = 10)
    private BigDecimal loanRatio;

    /** 放款金额 */
    @ApiModelProperty(value = "放款金额", position = 11)
    private BigDecimal loanAmount;

    /** 放款状态:已放款/放款中 */
    @ApiModelProperty(value = "放款状态", position = 12)
    private String loanState;

    /** 车量 */
    @ApiModelProperty(value = "车量", position = 13)
    private Integer loanTruckCount;

    /** 审批状态 */
    @ApiModelProperty(value = "二级审批状态", position = 15)
    private String bankOpApprovalState;

    /** 审批时间 */
    @ApiModelProperty(value = "审批时间", position = 16)
    private Date approvalTime;

    /** 审批信息 */
    @ApiModelProperty(value = "审批信息", position = 17)
    private String approvalMsg;

    /** 审批员 */
    @ApiModelProperty(value = "审批员", position = 18)
    private String approvalUser;
    @TableField(exist = false)
    private String approvalUserName;
    @TableField(exist = false)
    private String approvalUserAccount;

    public DepotLoanGrant() {
    }

    public DepotLoanGrant(String depotId) {
        this.depotId = depotId;
    }

    public DepotLoanGrant(String id, String bankOpApprovalState) {
        this.id = id;
        this.bankOpApprovalState = bankOpApprovalState;
    }

    public DepotLoanGrant(String id, String bankOpApprovalState, String approvalMsg, String loanState, BigDecimal loanPrice, BigDecimal loanRatio, BigDecimal loanAmount) {
        this.id = id;
        this.bankOpApprovalState = bankOpApprovalState;
        this.approvalMsg = approvalMsg;
        this.loanState = loanState;
        this.loanPrice = loanPrice;
        this.loanRatio = loanRatio;
        this.loanAmount = loanAmount;
    }
}
