/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotLoanOrder.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.core.Const.BizStr;
import vip.xiaonuo.biz.modular.depotWork.entity.DepotWork;
import vip.xiaonuo.biz.modular.depotWork.param.DepotWorkAddParam;
import vip.xiaonuo.biz.modular.depotWork.param.DepotWorkEditParam;
import vip.xiaonuo.biz.modular.depotWork.param.DepotWorkIdParam;
import vip.xiaonuo.biz.modular.depotWork.param.DepotWorkPageParam;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotLoanOrder.service.DepotLoanOrderService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 入库贷款控制器
 *
 * @author xiang
 * @date 2023/09/18 15:34
 */
@Api(tags = "入库贷款控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotLoanOrderController {

    @Resource
    private DepotLoanOrderService depotLoanOrderService;

    /**
     * 获取入库贷款分页
     *
     * @author xiang
     * @date 2023/09/18 15:34
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取入库贷款分页")
    @SaCheckPermission("/biz/depotLoanOrder/page")
    @GetMapping("/biz/depotLoanOrder/page")
    public CommonResult<Page<DepotWork>> page(DepotWorkPageParam param) {
        param.setDepotBizType(BizStr.KU_RU);
        return CommonResult.data(depotLoanOrderService.page(param));
    }

    /**
     * 添加入库贷款
     *
     * @author xiang
     * @date 2023/09/18 15:34
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加入库贷款")
    @CommonLog("添加入库贷款")
    @SaCheckPermission("/biz/depotLoanOrder/add")
    @PostMapping("/biz/depotLoanOrder/add")
    public CommonResult<String> add(@RequestBody @Valid DepotWorkAddParam param) {
        depotLoanOrderService.add(param);
        return CommonResult.ok();
    }

    /**
     * 编辑入库贷款
     *
     * @author xiang
     * @date 2023/09/18 15:34
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑入库贷款")
    @CommonLog("编辑入库贷款")
    @SaCheckPermission("/biz/depotLoanOrder/edit")
    @PostMapping("/biz/depotLoanOrder/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotWorkEditParam param) {
        depotLoanOrderService.edit(param);
        return CommonResult.ok();
    }

    /**
     * 删除入库贷款
     *
     * @author xiang
     * @date 2023/09/18 15:34
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除入库贷款")
    @CommonLog("删除入库贷款")
    @SaCheckPermission("/biz/depotLoanOrder/delete")
    @PostMapping("/biz/depotLoanOrder/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotWorkIdParam> idParamList) {
        depotLoanOrderService.delete(idParamList);
        return CommonResult.ok();
    }

    /**
     * 获取入库贷款详情
     *
     * @author xiang
     * @date 2023/09/18 15:34
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取入库贷款详情")
    @SaCheckPermission("/biz/depotLoanOrder/detail")
    @GetMapping("/biz/depotLoanOrder/detail")
    public CommonResult<DepotWork> detail(@Valid DepotWorkIdParam idParam) {
        return CommonResult.data(depotLoanOrderService.detail(idParam));
    }

    /**
     * 获取待放款详情
     *
     * @author xiang
     * @date 2024/01/16 15:34
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取待放款详情")
    @SaCheckPermission("/biz/depotLoanOrder/detail")
    @GetMapping("/biz/depotLoanOrder/detailGroup")
    public Object detailGroup(@Valid DepotWork param) {
        return CommonResult.data(depotLoanOrderService.detailGroup(param.getTruckBookId()));
    }
}
