/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotLockBook.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotLockBook.entity.DepotLockBook;
import vip.xiaonuo.biz.modular.depotLockBook.param.DepotLockBookAddParam;
import vip.xiaonuo.biz.modular.depotLockBook.param.DepotLockBookEditParam;
import vip.xiaonuo.biz.modular.depotLockBook.param.DepotLockBookIdParam;
import vip.xiaonuo.biz.modular.depotLockBook.param.DepotLockBookPageParam;
import vip.xiaonuo.biz.modular.depotLockBook.service.DepotLockBookService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 电子锁预约控制器
 *
 * @author xiang
 * @date 2023/09/14 09:44
 */
@Api(tags = "电子锁预约控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotLockBookController {

    @Resource
    private DepotLockBookService depotLockBookService;

    /**
     * 获取电子锁预约分页
     *
     * @author xiang
     * @date 2023/09/14 09:44
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取电子锁预约分页")
    @SaCheckPermission("/biz/depotLockBook/page")
    @GetMapping("/biz/depotLockBook/page")
    public CommonResult<Page<DepotLockBook>> page(DepotLockBookPageParam depotLockBookPageParam) {
        return CommonResult.data(depotLockBookService.page(depotLockBookPageParam));
    }

    /**
     * 添加电子锁预约
     *
     * @author xiang
     * @date 2023/09/14 09:44
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加电子锁预约")
    @CommonLog("添加电子锁预约")
    @SaCheckPermission("/biz/depotLockBook/add")
    @PostMapping("/biz/depotLockBook/add")
    public CommonResult<String> add(@RequestBody @Valid DepotLockBookAddParam depotLockBookAddParam) {
        depotLockBookService.add(depotLockBookAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑电子锁预约
     *
     * @author xiang
     * @date 2023/09/14 09:44
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑电子锁预约")
    @CommonLog("编辑电子锁预约")
    @SaCheckPermission("/biz/depotLockBook/edit")
    @PostMapping("/biz/depotLockBook/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotLockBookEditParam depotLockBookEditParam) {
        depotLockBookService.edit(depotLockBookEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除电子锁预约
     *
     * @author xiang
     * @date 2023/09/14 09:44
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除电子锁预约")
    @CommonLog("删除电子锁预约")
    @SaCheckPermission("/biz/depotLockBook/delete")
    @PostMapping("/biz/depotLockBook/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotLockBookIdParam> depotLockBookIdParamList) {
        depotLockBookService.delete(depotLockBookIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取电子锁预约详情
     *
     * @author xiang
     * @date 2023/09/14 09:44
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取电子锁预约详情")
    @SaCheckPermission("/biz/depotLockBook/detail")
    @GetMapping("/biz/depotLockBook/detail")
    public CommonResult<DepotLockBook> detail(@Valid DepotLockBookIdParam depotLockBookIdParam) {
        return CommonResult.data(depotLockBookService.detail(depotLockBookIdParam));
    }

    /**
     * 电子锁预约二级审批
     *
     * @author xiang
     * @date 2023/09/14 09:44
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("电子锁预约二级审批")
    @CommonLog("电子锁预约二级审批")
    @SaCheckPermission("/biz/depotLockBook/bankOpApproval")
    @PostMapping("/biz/depotLockBook/bankOpApproval")
    public CommonResult<String> bankOpApproval(@RequestBody @Valid DepotLockBookEditParam param) {
        depotLockBookService.edit(new DepotLockBookEditParam(param.getId(), param.getBankOpApprovalState(), param.getBankOpApprovalRemark()));
        return CommonResult.ok();
    }
}
