/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotRepayOrder.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.core.Const.BizStr;
import vip.xiaonuo.biz.modular.depotLoanGrant.entity.DepotLoanGrant;
import vip.xiaonuo.biz.modular.depotRepayOrder.param.*;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotRepayOrder.entity.DepotRepayOrder;
import vip.xiaonuo.biz.modular.depotRepayOrder.service.DepotRepayOrderService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 出库还款控制器
 *
 * @author xiang
 * @date 2023/09/18 16:45
 */
@Api(tags = "出库还款控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotRepayOrderController {

    @Resource
    private DepotRepayOrderService depotRepayOrderService;

    /**
     * 获取出库还款分页
     *
     * @author xiang
     * @date 2023/09/18 16:45
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取出库还款分页")
    @SaCheckPermission("/biz/depotRepayOrder/page")
    @GetMapping("/biz/depotRepayOrder/page")
    public CommonResult<Page<DepotRepayOrder>> page(DepotRepayOrderPageParam depotRepayOrderPageParam) {
        return CommonResult.data(depotRepayOrderService.page(depotRepayOrderPageParam));
    }

    /**
     * 添加出库还款
     *
     * @author xiang
     * @date 2023/09/18 16:45
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加出库还款")
    @CommonLog("添加出库还款")
    @SaCheckPermission("/biz/depotRepayOrder/add")
    @PostMapping("/biz/depotRepayOrder/add")
    public CommonResult<String> add(@RequestBody @Valid DepotRepayOrderAddParam depotRepayOrderAddParam) {
        depotRepayOrderService.add(depotRepayOrderAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑出库还款
     *
     * @author xiang
     * @date 2023/09/18 16:45
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑出库还款")
    @CommonLog("编辑出库还款")
    @SaCheckPermission("/biz/depotRepayOrder/edit")
    @PostMapping("/biz/depotRepayOrder/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotRepayOrderEditParam param) {
        depotRepayOrderService.edit(param);
        return CommonResult.ok();
    }

    /**
     * 删除出库还款
     *
     * @author xiang
     * @date 2023/09/18 16:45
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除出库还款")
    @CommonLog("删除出库还款")
    @SaCheckPermission("/biz/depotRepayOrder/delete")
    @PostMapping("/biz/depotRepayOrder/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotRepayOrderIdParam> depotRepayOrderIdParamList) {
        depotRepayOrderService.delete(depotRepayOrderIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取出库还款详情
     *
     * @author xiang
     * @date 2023/09/18 16:45
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取出库还款详情")
    @SaCheckPermission("/biz/depotRepayOrder/detail")
    @GetMapping("/biz/depotRepayOrder/detail")
    public CommonResult<Object> detail(@Valid DepotRepayOrderIdParam idParam) {
        return CommonResult.data("IdWorker".equals(idParam.getId()) ? new DepotRepayOrder(IdWorker.getIdStr(), BizStr.STATE_APPROVAL_N, null, null) : depotRepayOrderService.detail(idParam));
    }

    /**
     * 二级审批
     *
     * @author xiang
     * @date 2023/09/13 09:21
     */
    @ApiOperationSupport(order = 6)
    @ApiOperation("出库还款二级审批")
    @CommonLog("出库还款二级审批")
    @SaCheckPermission("/biz/depotRepayOrder/bankOpApproval")
    @PostMapping("/biz/depotRepayOrder/bankOpApproval")
    public CommonResult<String> bankOpApproval(@RequestBody @Valid DepotRepayOrderApprovalParam param) {
        depotRepayOrderService.bankOpApproval(param);
        return CommonResult.ok();
    }

    /**
     * 预约控制
     *
     * @author xiang
     * @date 2023/09/13 09:21
     */
    @ApiOperationSupport(order = 7)
    @ApiOperation("出库还款预约控制")
    @CommonLog("出库还款预约控制")
    @SaCheckPermission("/biz/depotRepayOrder/add")
    @PostMapping("/biz/depotRepayOrder/editBookFlag")
    public CommonResult<String> editBookFlag(@RequestBody @Valid DepotRepayOrderEditBookFlagParam param) {
        depotRepayOrderService.editBookFlag(param.getId(), param.getBookFlag());
        return CommonResult.ok();
    }

    /**
     * 出库还款添加候补车牌
     *
     * @author xiang
     * @date 2023/09/13 09:21
     */
    @ApiOperationSupport(order = 7)
    @ApiOperation("出库还款添加候补车牌")
    @CommonLog("出库还款添加候补车牌")
    @SaCheckPermission("/biz/depotRepayOrder/page")
    @PostMapping("/biz/depotRepayOrder/plug")
    public CommonResult<String> plug(@RequestBody @Valid DepotRepayOrderPlugParam param) {
        depotRepayOrderService.plug(param);
        return CommonResult.ok();
    }

    /**
     * 出库还款审批候补车牌
     *
     * @author xiang
     * @date 2023/09/13 09:21
     */
    @ApiOperationSupport(order = 7)
    @ApiOperation("出库还款添加候补车牌")
    @CommonLog("出库还款添加候补车牌")
    @SaCheckPermission("/biz/depotRepayOrder/bankOpApproval")
    @PostMapping("/biz/depotRepayOrder/approvalPlug")
    public CommonResult<String> approvalPlug(@RequestBody @Valid DepotRepayOrderPlugParam param) {
        depotRepayOrderService.approvalPlug(param);
        return CommonResult.ok();
    }

    /**
     * 关单
     *
     * @author xiang
     * @date 2023/09/13 09:21
     */
    @ApiOperationSupport(order = 7)
    @ApiOperation("出库还款手动关单")
    @CommonLog("出库还款手动关单")
    @SaCheckPermission("/biz/depotRepayOrder/offOrder")
    @PostMapping("/biz/depotRepayOrder/offOrder")
    public CommonResult<String> offOrder(@RequestBody @Valid DepotRepayOrderIdParam param) {
        depotRepayOrderService.offOrder(param.getId());
        return CommonResult.ok();
    }

    /**
     * 添加新单时获取并校验最近一次还款单
     *
     * @author xiang
     * @date 2023/09/18 16:45
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("出库还款添加新单时获取并校验最近一次还款单")
    @SaCheckPermission("/biz/depotRepayOrder/add")
    @GetMapping("/biz/depotRepayOrder/getBeforeAdd")
    public CommonResult<Object> getBeforeAdd(@Valid DepotRepayOrder order) {
        return CommonResult.data(depotRepayOrderService.getBeforeAdd(order.getDepotId(), order.getGrainGroup()));
    }

}
