/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotRepayOrder.param;

import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

import javax.validation.constraints.NotBlank;
import java.math.BigDecimal;

/**
 * 出库还款编辑参数
 *
 * @author xiang
 * @date  2023/09/18 16:45
 **/
@Getter
@Setter
public class DepotRepayOrderEditParam {

    /** ID */
    @ApiModelProperty(value = "ID", required = true, position = 1)
    @NotBlank(message = "id不能为空")
    private String id;

    /** 项目编码 */
    @ApiModelProperty(value = "项目编码", position = 2)
    private String depotId;

    /** 贷款合同号 */
    @ApiModelProperty(value = "贷款合同号", position = 3)
    private String contractCode;

    /** 还款本金 */
    @ApiModelProperty(value = "还款本金", position = 5)
    private BigDecimal repayPrincipal;

    /** 利息 */
    @ApiModelProperty(value = "利息", position = 6)
    private BigDecimal repayInterest;

    /** 还款单价 */
    @ApiModelProperty(value = "还款单价", position = 7)
    private BigDecimal repayPrice;

    /** 还款金额 */
    @ApiModelProperty(value = "还款金额", position = 7)
    private BigDecimal repayAmount;

    /** 本单出库量(吨):本金/还款单价 */
    @ApiModelProperty(value = "本单出库量(吨)", position = 8)
    private BigDecimal repayTonThis;

    /** 库位 */
    @ApiModelProperty(value = "库位", position = 8)
    private String depotShed;

    /** 总提货量(吨) */
    @ApiModelProperty(value = "总提货量(吨)", position = 9)
    private BigDecimal repayTonSum;

    /** 待出库重量(吨) */
    @ApiModelProperty(value = "待出库重量(吨)", position = 9)
    private BigDecimal repayTonSurplus;

    /** 上次扭转余量(吨) */
    @ApiModelProperty(value = "上次扭转余量(吨)", position = 9)
    private BigDecimal repayBeforeTonSurplus;

    /** 车牌 */
    @ApiModelProperty(value = "车牌", position = 10)
    private String licensePlate;

    /** 授权提货人 */
    @ApiModelProperty(value = "授权提货人", position = 11)
    private String keeperName;

    /** 授权提货人身份证号 */
    @ApiModelProperty(value = "授权提货人身份证号", position = 12)
    private String keeperCitizenId;

    /** 授权提货人身份证照片 */
    @ApiModelProperty(value = "授权提货人身份证照片", position = 13)
    private String keeperCitizenPic;

    /** 授权提货人联系方式 */
    @ApiModelProperty(value = "授权提货人联系方式", position = 14)
    private String keeperMobile;

    /** 二级审批状态 */
    @ApiModelProperty(value = "二级审批状态", position = 15)
    private String bankOpApprovalState;

    /** 预约状态:是/否 */
    @ApiModelProperty(value = "预约状态:是/否", position = 16)
    private String bookFlag;

    public DepotRepayOrderEditParam() {
    }

    public DepotRepayOrderEditParam(String id, String bookFlag) {
        this.id = id;
        this.bookFlag = bookFlag;
    }

    public DepotRepayOrderEditParam(String id, BigDecimal repayInterest, BigDecimal repayPrice, String bankOpApprovalState) {
        this.id = id;
        this.repayInterest = repayInterest;
        this.repayPrice = repayPrice;
        this.bankOpApprovalState = bankOpApprovalState;
    }
}
