/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotSampling.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotSampling.entity.DepotSampling;
import vip.xiaonuo.biz.modular.depotSampling.param.DepotSamplingAddParam;
import vip.xiaonuo.biz.modular.depotSampling.param.DepotSamplingEditParam;
import vip.xiaonuo.biz.modular.depotSampling.param.DepotSamplingIdParam;
import vip.xiaonuo.biz.modular.depotSampling.param.DepotSamplingPageParam;
import vip.xiaonuo.biz.modular.depotSampling.service.DepotSamplingService;
import vip.xiaonuo.common.util.DatesUtil;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 质价确认控制器
 *
 * @author xiang
 * @date 2023/09/13 16:37
 */
@Api(tags = "质价确认控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotSamplingController {

    @Resource
    private DepotSamplingService depotSamplingService;

    /**
     * 获取质价确认分页
     *
     * @author xiang
     * @date 2023/09/13 16:37
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取质价确认分页")
    @SaCheckPermission("/biz/depotSampling/page")
    @GetMapping("/biz/depotSampling/page")
    public CommonResult<Page<DepotSampling>> page(DepotSamplingPageParam depotSamplingPageParam) {
        return CommonResult.data(depotSamplingService.page(depotSamplingPageParam));
    }

    /**
     * 添加扦样
     *
     * @author xiang
     * @date 2023/09/13 16:37
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加扦样")
    @CommonLog("添加扦样")
    @SaCheckPermission("/biz/depotSampling/add")
    @PostMapping("/biz/depotSampling/addSampling")
    public CommonResult<String> addSampling(@RequestBody @Valid DepotSamplingAddParam param) {
        DepotSamplingAddParam addParam = new DepotSamplingAddParam(param.getTruckBookId(), param.getSamplingShuiFen()
                , param.getSamplingRongZhong(), param.getSamplingQueShao(), param.getSamplingMeiBian()
                , param.getSamplingZaZhi(), param.getSamplingPrice());
        addParam.setSamplingType(param.getSamplingType());
        depotSamplingService.addSampling(addParam);
        return CommonResult.ok();
    }

    /**
     * 编辑质价确认
     *
     * @author xiang
     * @date 2023/09/13 16:37
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑质价确认")
    @CommonLog("编辑质价确认")
    @SaCheckPermission("/biz/depotSampling/edit")
    @PostMapping("/biz/depotSampling/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotSamplingEditParam depotSamplingEditParam) {
        depotSamplingService.edit(depotSamplingEditParam);
        return CommonResult.ok();
    }

    /**
     * 二级审批确认
     *
     * @author xiang
     * @date 2023/09/13 09:21
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑签样单表二级审批确认")
    @CommonLog("编辑签样单表二级审批确认")
    @SaCheckPermission("/biz/depotSampling/bankOpApproval")
    @PostMapping("/biz/depotSampling/bankOpApproval")
    public CommonResult<String> bankOpApproval(@RequestBody @Valid DepotSamplingEditParam param) {
        depotSamplingService.edit(new DepotSamplingEditParam(param.getId(), param.getSaveState(), DatesUtil.forDateTime()));
        return CommonResult.ok();
    }

    /**
     * 删除质价确认
     *
     * @author xiang
     * @date 2023/09/13 16:37
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除质价确认")
    @CommonLog("删除质价确认")
    @SaCheckPermission("/biz/depotSampling/delete")
    @PostMapping("/biz/depotSampling/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotSamplingIdParam> depotSamplingIdParamList) {
        depotSamplingService.delete(depotSamplingIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取质价确认详情
     *
     * @author xiang
     * @date 2023/09/13 16:37
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取质价确认详情")
    @SaCheckPermission("/biz/depotSampling/detail")
    @GetMapping("/biz/depotSampling/detail")
    public CommonResult<DepotSampling> detail(@Valid DepotSamplingIdParam depotSamplingIdParam) {
        return CommonResult.data(depotSamplingService.detail(depotSamplingIdParam));
    }
}
