/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotTruckBook.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.modular.depotTruckBook.entity.DepotTruckBook;
import vip.xiaonuo.biz.modular.depotTruckBook.param.*;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotTruckBook.service.DepotTruckBookService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 车辆预约表控制器
 *
 * @author xiang
 * @date 2023/09/13 09:21
 */
@Api(tags = "车辆预约表控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotTruckBookController {

    @Resource
    private DepotTruckBookService depotTruckBookService;

    /**
     * 获取车辆预约表分页
     *
     * @author xiang
     * @date 2023/09/13 09:21
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取车辆预约表分页")
    @SaCheckPermission("/biz/depotTruckBook/page")
    @GetMapping("/biz/depotTruckBook/page")
    public CommonResult<Page<DepotTruckBook>> page(DepotTruckBookPageParam depotTruckBookPageParam) {
        return CommonResult.data(depotTruckBookService.page(depotTruckBookPageParam));
    }

    /**
     * 添加车辆预约表
     *
     * @author xiang
     * @date 2023/09/13 09:21
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加车辆预约表")
    @CommonLog("添加车辆预约表")
    @SaCheckPermission("/biz/depotTruckBook/add")
    @PostMapping("/biz/depotTruckBook/add")
    public CommonResult<String> add(@RequestBody @Valid DepotTruckBookAddParam depotTruckBookAddParam) {
        depotTruckBookService.add(depotTruckBookAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑车辆预约表
     *
     * @author xiang
     * @date 2023/09/13 09:21
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑车辆预约表")
    @CommonLog("编辑车辆预约表")
    @SaCheckPermission("/biz/depotTruckBook/edit")
    @PostMapping("/biz/depotTruckBook/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotTruckBookEditParam depotTruckBookEditParam) {
        depotTruckBookService.edit(depotTruckBookEditParam);
        return CommonResult.ok();
    }

    /**
     * 一级审批
     *
     * @author xiang
     * @date 2023/09/13 09:21
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑车辆预约表一级审批")
    @CommonLog("编辑车辆预约表一级审批")
    @SaCheckPermission("/biz/depotTruckBook/depotOpApproval")
    @PostMapping("/biz/depotTruckBook/depotOpApproval")
    public CommonResult<String> depotOpApproval(@RequestBody @Valid DepotTruckBookApprovalParam param) {
        depotTruckBookService.depotOpApproval(param);
        return CommonResult.ok();
    }

    /**
     * 删除车辆预约表
     *
     * @author xiang
     * @date 2023/09/13 09:21
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除车辆预约表")
    @CommonLog("删除车辆预约表")
    @SaCheckPermission("/biz/depotTruckBook/delete")
    @PostMapping("/biz/depotTruckBook/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotTruckBookIdParam> depotTruckBookIdParamList) {
        depotTruckBookService.delete(depotTruckBookIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取车辆预约表详情
     *
     * @author xiang
     * @date 2023/09/13 09:21
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取车辆预约表详情")
    @SaCheckPermission("/biz/depotTruckBook/detail")
    @GetMapping("/biz/depotTruckBook/detail")
    public CommonResult<DepotTruckBook> detail(@Valid DepotTruckBookIdParam depotTruckBookIdParam) {
        return CommonResult.data(depotTruckBookService.detail(depotTruckBookIdParam));
    }
}
