/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotTruckBook.param;

import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import vip.xiaonuo.biz.core.Const.BizStr;

import javax.validation.constraints.NotBlank;

/**
 * 车辆预约表编辑参数
 *
 * @author xiang
 * @date  2023/09/13 09:21
 **/
@Getter
@Setter
public class DepotTruckBookEditParam {

    /** ID */
    @ApiModelProperty(value = "ID", required = true, position = 1)
    @NotBlank(message = "id不能为空")
    private String id;

    /** 项目编码 */
    @ApiModelProperty(value = "项目编码", position = 2)
    private String depotId;

    /** 出入库:入库/出库 */
    @ApiModelProperty(value = "出入库:入库/出库", position = 3)
    private String depotBizType;

    /** 车牌 */
    @ApiModelProperty(value = "车牌", position = 4)
    private String licensePlate;

    /** 司机姓名 */
    @ApiModelProperty(value = "司机姓名", position = 5)
    private String driverName;

    /** 司机手机号 */
    @ApiModelProperty(value = "司机手机号", position = 6)
    private String driverMobile;

    /** 司机行驶证 */
    @ApiModelProperty(value = "司机行驶证", position = 7)
    private String driverTravel;

    /** 司机驾驶证 */
    @ApiModelProperty(value = "司机驾驶证", position = 8)
    private String driverLicense;

    /** 经办人身份证号 */
    @ApiModelProperty(value = "经办人身份证号", position = 8)
    private String keeperCitizenId;

    /** 企业单位 */
    @ApiModelProperty(value = "企业单位", position = 9)
    private String companyName;

    /** 企业单位联系人 */
    @ApiModelProperty(value = "企业单位联系人", position = 10)
    private String companySalesmanName;

    /** 企业单位联系人电话 */
    @ApiModelProperty(value = "企业单位联系人电话", position = 11)
    private String companySalesmanMobile;

    /** 粮食种类 */
    @ApiModelProperty(value = "粮食种类", position = 12)
    private String grainGroup;

    /** 粮食类型 */
    @ApiModelProperty(value = "粮食类型", position = 13)
    private String grainType;

    /** 预约日期 */
    @ApiModelProperty(value = "预约日期", position = 19)
    private String bookDate;

    /** 收款人 */
    @ApiModelProperty(value = "收款人", position = 14)
    private String payeeName;

    /** 收款人手机号 */
    @ApiModelProperty(value = "收款人手机号", position = 15)
    private String payeeMobile;

    /** 收款人开户行 */
    @ApiModelProperty(value = "收款人开户行", position = 16)
    private String payeeBank;

    /** 收款人开户行账户 */
    @ApiModelProperty(value = "收款人开户行账户", position = 17)
    private String payeeBankBillNo;

    /** 预约状态:正常预约/已结束/超时作废 */
    @ApiModelProperty(value = "预约状态:正常预约/已结束/超时作废", position = 18)
    private String bookState;

    /** 粮库管理员审批状态:待审批/已审批/未通过 */
    @ApiModelProperty(value = "预约状态:待审批/已审批/未通过", position = 20)
    private String depotOpApprovalState;

    /** 粮库管理员审批信息 */
    @ApiModelProperty(value = "粮库管理员审批信息", position = 20)
    private String depotOpApprovalMsg;

    /** 粮库管理员审批时间 */
    @ApiModelProperty(value = "粮库管理员审批时间", position = 20)
    private String depotOpApprovalTime;

    /** 银行管理员审批状态:待审批/已审批/未通过 */
    @ApiModelProperty(value = "预约状态:待审批/已审批/未通过", position = 21)
    private String bankOpApprovalState;

    /** 银行管理员审批信息 */
    @ApiModelProperty(value = "银行管理员审批信息", position = 21)
    private String bankOpApprovalMsg;

    /** 银行管理员审批时间 */
    @ApiModelProperty(value = "银行管理员审批时间", position = 21)
    private String bankOpApprovalTime;

    /** 流程节点 */
    @ApiModelProperty(value = "流程节点", position = 22)
    private String bizFlowName;

    public DepotTruckBookEditParam() {
    }

    public DepotTruckBookEditParam(String id, String bizFlowName) {
        this.id = id;
        this.bizFlowName = bizFlowName;
    }

    /** 管理员审批状态:待审批/已审批/未通过 */
    public DepotTruckBookEditParam(String id, String depotOpApprovalState, String bankOpApprovalState) {
        this.id = id;
        this.depotOpApprovalState = depotOpApprovalState;
        this.bankOpApprovalState = bankOpApprovalState;
    }

    public DepotTruckBookEditParam(String id, String depotOpApprovalState, String depotOpApprovalMsg, String depotOpApprovalTime, String bankOpApprovalState, String bankOpApprovalMsg, String bankOpApprovalTime) {
        this.id = id;
        this.depotOpApprovalState = depotOpApprovalState;
        this.depotOpApprovalMsg = depotOpApprovalMsg;
        this.depotOpApprovalTime = depotOpApprovalTime;
        this.bankOpApprovalState = bankOpApprovalState;
        this.bankOpApprovalMsg = bankOpApprovalMsg;
        this.bankOpApprovalTime = bankOpApprovalTime;
    }
}
