/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotTruckWeighDouble.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.modular.depotTruckWeighDouble.param.*;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotTruckWeighDouble.entity.DepotTruckWeighDouble;
import vip.xiaonuo.biz.modular.depotTruckWeighDouble.service.DepotTruckWeighDoubleService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 双次过磅记录表控制器
 *
 * @author xiang
 * @date 2023/10/08 17:05
 */
@Api(tags = "双次过磅记录表控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotTruckWeighDoubleController {

    @Resource
    private DepotTruckWeighDoubleService depotTruckWeighDoubleService;

    /**
     * 获取双次过磅记录表分页
     *
     * @author xiang
     * @date 2023/10/08 17:05
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取双次过磅记录表分页")
    @SaCheckPermission("/biz/depotTruckWeighDouble/page")
    @GetMapping("/biz/depotTruckWeighDouble/page")
    public CommonResult<Page<DepotTruckWeighDouble>> page(DepotTruckWeighDoublePageParam depotTruckWeighDoublePageParam) {
        return CommonResult.data(depotTruckWeighDoubleService.page(depotTruckWeighDoublePageParam));
    }

    /**
     * 添加双次过磅记录表
     *
     * @author xiang
     * @date 2023/10/08 17:05
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加双次过磅记录表")
    @CommonLog("添加双次过磅记录表")
    @SaCheckPermission("/biz/depotTruckWeighDouble/add")
    @PostMapping("/biz/depotTruckWeighDouble/add")
    public CommonResult<String> add(@RequestBody @Valid DepotTruckWeighDoubleAddParam param) {
        depotTruckWeighDoubleService.add(param);
        return CommonResult.ok();
    }

    /**
     * 编辑双次过磅记录表
     *
     * @author xiang
     * @date 2023/10/08 17:05
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑双次过磅记录表")
    @CommonLog("编辑双次过磅记录表")
    @SaCheckPermission("/biz/depotTruckWeighDouble/edit")
    @PostMapping("/biz/depotTruckWeighDouble/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotTruckWeighDoubleEditParam depotTruckWeighDoubleEditParam) {
        depotTruckWeighDoubleService.edit(depotTruckWeighDoubleEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除双次过磅记录表
     *
     * @author xiang
     * @date 2023/10/08 17:05
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除双次过磅记录表")
    @CommonLog("删除双次过磅记录表")
    @SaCheckPermission("/biz/depotTruckWeighDouble/delete")
    @PostMapping("/biz/depotTruckWeighDouble/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotTruckWeighDoubleIdParam> depotTruckWeighDoubleIdParamList) {
        depotTruckWeighDoubleService.delete(depotTruckWeighDoubleIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取双次过磅记录表详情
     *
     * @author xiang
     * @date 2023/10/08 17:05
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取双次过磅记录表详情")
    @SaCheckPermission("/biz/depotTruckWeighDouble/detail")
    @GetMapping("/biz/depotTruckWeighDouble/detail")
    public CommonResult<DepotTruckWeighDouble> detail(@Valid DepotTruckWeighDouble param) {
        return CommonResult.data(depotTruckWeighDoubleService.detail(param.getId(), param.getTruckBookId()));
    }
}
