/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotTruckWeighDoublePic.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotTruckWeighDoublePic.entity.DepotTruckWeighDoublePic;
import vip.xiaonuo.biz.modular.depotTruckWeighDoublePic.param.DepotTruckWeighDoublePicAddParam;
import vip.xiaonuo.biz.modular.depotTruckWeighDoublePic.param.DepotTruckWeighDoublePicEditParam;
import vip.xiaonuo.biz.modular.depotTruckWeighDoublePic.param.DepotTruckWeighDoublePicIdParam;
import vip.xiaonuo.biz.modular.depotTruckWeighDoublePic.param.DepotTruckWeighDoublePicPageParam;
import vip.xiaonuo.biz.modular.depotTruckWeighDoublePic.service.DepotTruckWeighDoublePicService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 过磅记录图片表控制器
 *
 * @author xiang
 * @date 2023/10/10 17:32
 */
@Api(tags = "过磅记录图片表控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotTruckWeighDoublePicController {

    @Resource
    private DepotTruckWeighDoublePicService depotTruckWeighDoublePicService;

    /**
     * 获取过磅记录图片表分页
     *
     * @author xiang
     * @date 2023/10/10 17:32
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取过磅记录图片表分页")
    @SaCheckPermission("/biz/depotTruckWeighDoublePic/page")
    @GetMapping("/biz/depotTruckWeighDoublePic/page")
    public CommonResult<Page<DepotTruckWeighDoublePic>> page(DepotTruckWeighDoublePicPageParam depotTruckWeighDoublePicPageParam) {
        return CommonResult.data(depotTruckWeighDoublePicService.page(depotTruckWeighDoublePicPageParam));
    }

    /**
     * 添加过磅记录图片表
     *
     * @author xiang
     * @date 2023/10/10 17:32
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加过磅记录图片表")
    @CommonLog("添加过磅记录图片表")
    @SaCheckPermission("/biz/depotTruckWeighDoublePic/add")
    @PostMapping("/biz/depotTruckWeighDoublePic/add")
    public CommonResult<String> add(@RequestBody @Valid DepotTruckWeighDoublePicAddParam depotTruckWeighDoublePicAddParam) {
        depotTruckWeighDoublePicService.add(depotTruckWeighDoublePicAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑过磅记录图片表
     *
     * @author xiang
     * @date 2023/10/10 17:32
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑过磅记录图片表")
    @CommonLog("编辑过磅记录图片表")
    @SaCheckPermission("/biz/depotTruckWeighDoublePic/edit")
    @PostMapping("/biz/depotTruckWeighDoublePic/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotTruckWeighDoublePicEditParam depotTruckWeighDoublePicEditParam) {
        depotTruckWeighDoublePicService.edit(depotTruckWeighDoublePicEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除过磅记录图片表
     *
     * @author xiang
     * @date 2023/10/10 17:32
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除过磅记录图片表")
    @CommonLog("删除过磅记录图片表")
    @SaCheckPermission("/biz/depotTruckWeighDoublePic/delete")
    @PostMapping("/biz/depotTruckWeighDoublePic/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotTruckWeighDoublePicIdParam> depotTruckWeighDoublePicIdParamList) {
        depotTruckWeighDoublePicService.delete(depotTruckWeighDoublePicIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取过磅记录图片表详情
     *
     * @author xiang
     * @date 2023/10/10 17:32
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取过磅记录图片表详情")
    @SaCheckPermission("/biz/depotTruckWeighDoublePic/detail")
    @GetMapping("/biz/depotTruckWeighDoublePic/detail")
    public CommonResult<DepotTruckWeighDoublePic> detail(@Valid DepotTruckWeighDoublePicIdParam depotTruckWeighDoublePicIdParam) {
        return CommonResult.data(depotTruckWeighDoublePicService.detail(depotTruckWeighDoublePicIdParam));
    }
}
