/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotUserOnlineLog.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotUserOnlineLog.entity.DepotUserOnlineLog;
import vip.xiaonuo.biz.modular.depotUserOnlineLog.param.DepotUserOnlineLogAddParam;
import vip.xiaonuo.biz.modular.depotUserOnlineLog.param.DepotUserOnlineLogEditParam;
import vip.xiaonuo.biz.modular.depotUserOnlineLog.param.DepotUserOnlineLogIdParam;
import vip.xiaonuo.biz.modular.depotUserOnlineLog.param.DepotUserOnlineLogPageParam;
import vip.xiaonuo.biz.modular.depotUserOnlineLog.service.DepotUserOnlineLogService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 人员签到控制器
 *
 * @author xiang
 * @date 2023/09/27 19:31
 */
@Api(tags = "人员签到控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotUserOnlineLogController {

    @Resource
    private DepotUserOnlineLogService depotUserOnlineLogService;

    /**
     * 获取人员签到分页
     *
     * @author xiang
     * @date 2023/09/27 19:31
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取人员签到分页")
    @SaCheckPermission("/biz/depotUserOnlineLog/page")
    @GetMapping("/biz/depotUserOnlineLog/page")
    public CommonResult<Page<DepotUserOnlineLog>> page(DepotUserOnlineLogPageParam depotUserOnlineLogPageParam) {
        Page<DepotUserOnlineLog> page = depotUserOnlineLogService.page(depotUserOnlineLogPageParam);
        return CommonResult.data(page);
    }

    /**
     * 添加人员签到
     *
     * @author xiang
     * @date 2023/09/27 19:31
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加人员签到")
    @CommonLog("添加人员签到")
    @SaCheckPermission("/biz/depotUserOnlineLog/add")
    @PostMapping("/biz/depotUserOnlineLog/add")
    public CommonResult<String> add(@RequestBody @Valid DepotUserOnlineLogAddParam depotUserOnlineLogAddParam) {
        depotUserOnlineLogService.add(depotUserOnlineLogAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑人员签到
     *
     * @author xiang
     * @date 2023/09/27 19:31
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑人员签到")
    @CommonLog("编辑人员签到")
    @SaCheckPermission("/biz/depotUserOnlineLog/edit")
    @PostMapping("/biz/depotUserOnlineLog/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotUserOnlineLogEditParam depotUserOnlineLogEditParam) {
        depotUserOnlineLogService.edit(depotUserOnlineLogEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除人员签到
     *
     * @author xiang
     * @date 2023/09/27 19:31
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除人员签到")
    @CommonLog("删除人员签到")
    @SaCheckPermission("/biz/depotUserOnlineLog/delete")
    @PostMapping("/biz/depotUserOnlineLog/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotUserOnlineLogIdParam> depotUserOnlineLogIdParamList) {
        depotUserOnlineLogService.delete(depotUserOnlineLogIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取人员签到详情
     *
     * @author xiang
     * @date 2023/09/27 19:31
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取人员签到详情")
    @SaCheckPermission("/biz/depotUserOnlineLog/detail")
    @GetMapping("/biz/depotUserOnlineLog/detail")
    public CommonResult<DepotUserOnlineLog> detail(@Valid DepotUserOnlineLogIdParam depotUserOnlineLogIdParam) {
        return CommonResult.data(depotUserOnlineLogService.detail(depotUserOnlineLogIdParam));
    }
}
