/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotUserOnlineLog.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.biz.modular.depotGrain.service.DepotGrainService;
import vip.xiaonuo.biz.modular.user.service.BizUserService;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.biz.modular.depotUserOnlineLog.entity.DepotUserOnlineLog;
import vip.xiaonuo.biz.modular.depotUserOnlineLog.mapper.DepotUserOnlineLogMapper;
import vip.xiaonuo.biz.modular.depotUserOnlineLog.param.DepotUserOnlineLogAddParam;
import vip.xiaonuo.biz.modular.depotUserOnlineLog.param.DepotUserOnlineLogEditParam;
import vip.xiaonuo.biz.modular.depotUserOnlineLog.param.DepotUserOnlineLogIdParam;
import vip.xiaonuo.biz.modular.depotUserOnlineLog.param.DepotUserOnlineLogPageParam;
import vip.xiaonuo.common.prop.CommonProperties;
import vip.xiaonuo.dev.api.DevFileApi;
import vip.xiaonuo.dev.modular.file.util.MultipartFileUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 人员签到Service接口实现类
 *
 * @author xiang
 * @date 2023/09/27 19:31
 **/
@Service
public class DepotUserOnlineLogService extends ServiceImpl<DepotUserOnlineLogMapper, DepotUserOnlineLog> {

    @Resource
    BizUserService bizUserService;
    @Resource
    DevFileApi devFileApi;
    @Resource
    DepotGrainService depotGrainService;
    @Resource
    private CommonProperties commonProperties;

    public Page<DepotUserOnlineLog> page(DepotUserOnlineLogPageParam param) {
        QueryWrapper<DepotUserOnlineLog> queryWrapper = new QueryWrapper<>(BeanUtil.toBean(param, DepotUserOnlineLog.class));
        List<String> depotIdLst = depotGrainService.lstOrgGrep();
        queryWrapper.lambda().in(CollUtil.isNotEmpty(depotIdLst), DepotUserOnlineLog::getDepotId, depotIdLst);
        if (ObjectUtil.isAllNotEmpty(param.getSortField(), param.getSortOrder())) {
            CommonSortOrderEnum.validate(param.getSortOrder());
            queryWrapper.orderBy(true, param.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(param.getSortField()));
        } else {
            queryWrapper.lambda().orderByDesc(DepotUserOnlineLog::getId);
        }
        Page page = this.page(CommonPageRequest.defaultPage(), queryWrapper);
        //匹配粮库
        depotGrainService.putDepotName(page.getRecords());
        //匹配用户名称
        bizUserService.putUserName(page.getRecords());
        //匹配完整图片地址
        putPicUrl(page.getRecords());
        return page;
    }

    public void add(DepotUserOnlineLogAddParam param) {
        DepotUserOnlineLog log = BeanUtil.toBean(param, DepotUserOnlineLog.class);
        log.setOnlinePicUrl(devFileApi.storageFilePathReturnUrl(MultipartFileUtils.base64ToMultipart(param.getFile())));
        this.save(log);
    }

    public void edit(DepotUserOnlineLogEditParam depotUserOnlineLogEditParam) {
        DepotUserOnlineLog depotUserOnlineLog = this.queryEntity(depotUserOnlineLogEditParam.getId());
        BeanUtil.copyProperties(depotUserOnlineLogEditParam, depotUserOnlineLog);
        this.updateById(depotUserOnlineLog);
    }

    @Transactional(rollbackFor = Exception.class)
    public void delete(List<DepotUserOnlineLogIdParam> depotUserOnlineLogIdParamList) {
        // 执行删除
        this.removeBatchByIds(CollStreamUtil.toList(depotUserOnlineLogIdParamList, DepotUserOnlineLogIdParam::getId));
    }

    public DepotUserOnlineLog detail(DepotUserOnlineLogIdParam depotUserOnlineLogIdParam) {
        return this.queryEntity(depotUserOnlineLogIdParam.getId());
    }

    public DepotUserOnlineLog queryEntity(String id) {
        DepotUserOnlineLog depotUserOnlineLog = this.getById(id);
        if (ObjectUtil.isEmpty(depotUserOnlineLog)) {
            throw new CommonException("人员签到不存在，id值为：{}", id);
        }
        return depotUserOnlineLog;
    }

    /**
     * 匹配完整图片地址
     *
     * @author xiang
     * @date 2023/09/27 19:31
     **/
    public void putPicUrl(List<DepotUserOnlineLog> list) {
        list.forEach(one -> {
            one.setOnlinePicUrl(commonProperties.getBackendUrl() + one.getOnlinePicUrl());
        });
    }
}
