/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotWork.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotWork.entity.DepotWork;
import vip.xiaonuo.biz.modular.depotWork.param.DepotWorkAddParam;
import vip.xiaonuo.biz.modular.depotWork.param.DepotWorkEditParam;
import vip.xiaonuo.biz.modular.depotWork.param.DepotWorkIdParam;
import vip.xiaonuo.biz.modular.depotWork.param.DepotWorkPageParam;
import vip.xiaonuo.biz.modular.depotWork.service.DepotWorkService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 出入库管理表控制器
 *
 * @author xiang
 * @date  2023/09/18 10:20
 */
@Api(tags = "出入库管理表控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotWorkController {

    @Resource
    private DepotWorkService depotWorkService;

    /**
     * 获取出入库管理表分页
     *
     * @author xiang
     * @date  2023/09/18 10:20
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取出入库管理表分页")
    @SaCheckPermission("/biz/depotWork/page")
    @GetMapping("/biz/depotWork/page")
    public CommonResult<Page<DepotWork>> page(DepotWorkPageParam depotWorkPageParam) {
        return CommonResult.data(depotWorkService.page(depotWorkPageParam));
    }

    /**
     * 获取出入库管理表分页
     *
     * @author xiang
     * @date  2023/09/18 10:20
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取出入库管理表分页")
    @SaCheckPermission("/biz/depotWork/page")
    @GetMapping("/biz/depotWork/list")
    public CommonResult<List<DepotWork>> list(DepotWorkPageParam depotWorkPageParam) {
        return CommonResult.data(depotWorkService.lst(depotWorkPageParam));
    }

    /**
     * 添加出入库管理表
     *
     * @author xiang
     * @date  2023/09/18 10:20
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加出入库管理表")
    @CommonLog("添加出入库管理表")
    @SaCheckPermission("/biz/depotWork/add")
    @PostMapping("/biz/depotWork/add")
    public CommonResult<String> add(@RequestBody @Valid DepotWorkAddParam depotWorkAddParam) {
        depotWorkService.add(depotWorkAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑出入库管理表
     *
     * @author xiang
     * @date  2023/09/18 10:20
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑出入库管理表")
    @CommonLog("编辑出入库管理表")
    @SaCheckPermission("/biz/depotWork/edit")
    @PostMapping("/biz/depotWork/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotWorkEditParam depotWorkEditParam) {
        depotWorkService.edit(depotWorkEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除出入库管理表
     *
     * @author xiang
     * @date  2023/09/18 10:20
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除出入库管理表")
    @CommonLog("删除出入库管理表")
    @SaCheckPermission("/biz/depotWork/delete")
    @PostMapping("/biz/depotWork/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<DepotWorkIdParam> depotWorkIdParamList) {
        depotWorkService.delete(depotWorkIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取出入库管理表详情
     *
     * @author xiang
     * @date  2023/09/18 10:20
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取出入库管理表详情")
    @SaCheckPermission("/biz/depotWork/detail")
    @GetMapping("/biz/depotWork/detail")
    public CommonResult<DepotWork> detail(@Valid DepotWorkIdParam depotWorkIdParam) {
        return CommonResult.data(depotWorkService.detail(depotWorkIdParam));
    }
}
