/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotWork.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import vip.xiaonuo.biz.modular.depotGrain.entity.BaseDepotGrainName;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 出入库管理表实体
 *
 * @author xiang
 * @date  2023/09/18 10:20
 **/
@Getter
@Setter
@TableName("DEPOT_WORK")
public class DepotWork extends BaseDepotGrainName {

    /** ID */
    @TableId
    @ApiModelProperty(value = "ID", position = 1)
    private String id;

    /** 项目编码 */
    @ApiModelProperty(value = "项目编码", position = 2)
    private String depotId;

    /** 车辆预约ID */
    @ApiModelProperty(value = "车辆预约ID", position = 3)
    private String truckBookId;

    /** 还款单号 */
    @ApiModelProperty(value = "还款单号", position = 3)
    private String repayOrderNo;

    /** 放款单号 */
    @ApiModelProperty(value = "放款单号", position = 3)
    private String loanOrderNo;

    /** 放款状态:待放款/已放款/放款中 */
    @ApiModelProperty(value = "放款状态", position = 3)
    private String loanState;

    /** 出入库 */
    @ApiModelProperty(value = "出入库", position = 4)
    private String depotBizType;

    /** 粮食种类 */
    @ApiModelProperty(value = "粮食种类", position = 5)
    private String grainGroup;

    /** 粮食类型 */
    @ApiModelProperty(value = "粮食类型", position = 5)
    private String grainType;

    /** 单据分类:系统自动/人工补单 */
    @ApiModelProperty(value = "单据分类:系统自动/人工补单", position = 5)
    private String billGroup;

    /** 负责人身份证号 */
    @ApiModelProperty(value = "负责人身份证号", position = 5)
    private String keeperCitizenId;

    /** 出入库数量（吨） */
    @ApiModelProperty(value = "出入库数量（吨）", position = 6)
    private BigDecimal depotTonCount;

    /** 出入库价格 */
    @ApiModelProperty(value = "出入库价格", position = 7)
    private BigDecimal depotPrice;

    /** 总价 */
    @ApiModelProperty(value = "总价", position = 8)
    private BigDecimal orderAmount;

    /** 干粮数量（吨） */
    @ApiModelProperty(value = "干粮数量（吨）", position = 6)
    private BigDecimal dryTonCount;

    /** 干粮价格（元/吨） */
    @ApiModelProperty(value = "干粮价格（元/吨）", position = 7)
    private BigDecimal dryPrice;

    /** 质量 */
    @ApiModelProperty(value = "质量", position = 9)
    private String depotQuality;

    /** 车牌 */
    @ApiModelProperty(value = "车牌", position = 10)
    private String licensePlate;

    /** 车次编码：yyyymmdd**** */
    @ApiModelProperty(value = "车次编码：yyyymmdd****", position = 11)
    private String carNumber;

    /** 过毛时间 */
    @ApiModelProperty(value = "过毛时间", position = 12)
    private Date grossTime;

    /** 过皮时间 */
    @ApiModelProperty(value = "过皮时间", position = 13)
    private Date tareTime;

    /** 毛重(公斤) */
    @ApiModelProperty(value = "毛重(公斤)", position = 15)
    private BigDecimal grossWeight;

    /** 皮重(公斤) */
    @ApiModelProperty(value = "皮重(公斤)", position = 16)
    private BigDecimal tareWeight;

    /** 净重(公斤) */
    @ApiModelProperty(value = "净重(公斤)", position = 17)
    private BigDecimal netWeight;

    /** 潮干比 */
    @ApiModelProperty(value = "潮干比", position = 18)
    private BigDecimal dampRatio;

    /** 干粮水份(%) */
    @ApiModelProperty(value = "干粮水份(%)", position = 18)
    private BigDecimal canonRatio;

    /** 放款审批员 */
    @TableField(exist = false)
    private String approvalUserName;
    @TableField(exist = false)
    private String approvalUserAccount;

    public DepotWork() {
    }

    public DepotWork(String depotId) {
        this.depotId = depotId;
    }

    public DepotWork(String repayOrderNo, String loanOrderNo) {
        this.repayOrderNo = repayOrderNo;
        this.loanOrderNo = loanOrderNo;
    }
}
