/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotWorkAlarm.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotWorkAlarm.entity.DepotWorkAlarm;
import vip.xiaonuo.biz.modular.depotWorkAlarm.param.DepotWorkAlarmAddParam;
import vip.xiaonuo.biz.modular.depotWorkAlarm.param.DepotWorkAlarmEditParam;
import vip.xiaonuo.biz.modular.depotWorkAlarm.param.DepotWorkAlarmIdParam;
import vip.xiaonuo.biz.modular.depotWorkAlarm.param.DepotWorkAlarmPageParam;
import vip.xiaonuo.biz.modular.depotWorkAlarm.service.DepotWorkAlarmService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 出入库异常控制器
 *
 * @author xiang
 * @date 2023/09/18 11:09
 */
@Api(tags = "出入库异常控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotWorkAlarmController {

    @Resource
    private DepotWorkAlarmService depotWorkAlarmService;

    /**
     * 获取出入库异常分页
     *
     * @author xiang
     * @date 2023/09/18 11:09
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取出入库异常分页")
    @SaCheckPermission("/biz/depotWorkAlarm/page")
    @GetMapping("/biz/depotWorkAlarm/page")
    public CommonResult<Page<DepotWorkAlarm>> page(DepotWorkAlarmPageParam depotWorkAlarmPageParam) {
        return CommonResult.data(depotWorkAlarmService.page(depotWorkAlarmPageParam));
    }

    /**
     * 添加出入库异常
     *
     * @author xiang
     * @date 2023/09/18 11:09
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加出入库异常")
    @CommonLog("添加出入库异常")
    @SaCheckPermission("/biz/depotWorkAlarm/add")
    @PostMapping("/biz/depotWorkAlarm/add")
    public CommonResult<String> add(@RequestBody @Valid DepotWorkAlarmAddParam depotWorkAlarmAddParam) {
        depotWorkAlarmService.add(depotWorkAlarmAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑出入库异常
     *
     * @author xiang
     * @date 2023/09/18 11:09
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑出入库异常")
    @CommonLog("编辑出入库异常")
    @SaCheckPermission("/biz/depotWorkAlarm/edit")
    @PostMapping("/biz/depotWorkAlarm/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotWorkAlarmEditParam depotWorkAlarmEditParam) {
        depotWorkAlarmService.edit(depotWorkAlarmEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除出入库异常
     *
     * @author xiang
     * @date 2023/09/18 11:09
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除出入库异常")
    @CommonLog("删除出入库异常")
    @SaCheckPermission("/biz/depotWorkAlarm/delete")
    @PostMapping("/biz/depotWorkAlarm/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotWorkAlarmIdParam> depotWorkAlarmIdParamList) {
        depotWorkAlarmService.delete(depotWorkAlarmIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取出入库异常详情
     *
     * @author xiang
     * @date 2023/09/18 11:09
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取出入库异常详情")
    @SaCheckPermission("/biz/depotWorkAlarm/detail")
    @GetMapping("/biz/depotWorkAlarm/detail")
    public CommonResult<DepotWorkAlarm> detail(@Valid DepotWorkAlarmIdParam depotWorkAlarmIdParam) {
        return CommonResult.data(depotWorkAlarmService.detail(depotWorkAlarmIdParam));
    }

    /**
     * 出入库异常处理
     *
     * @author xiang
     * @date 2023/09/18 11:09
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取出入库异常详情")
    @SaCheckPermission("/biz/depotWorkAlarm/detail")
    @PostMapping("/biz/depotWorkAlarm/processTo")
    public CommonResult<DepotWorkAlarm> processTo(@RequestBody DepotWorkAlarm param) {
        depotWorkAlarmService.processTo(param);
        return CommonResult.ok();
    }
}
