/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotWorkAlarmSet.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.logging.log4j.util.Strings;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.biz.core.Const.BizStr;
import vip.xiaonuo.biz.core.exception.BizExceptionEnum;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.biz.modular.depotWorkAlarmSet.entity.DepotWorkAlarmSet;
import vip.xiaonuo.biz.modular.depotWorkAlarmSet.mapper.DepotWorkAlarmSetMapper;
import vip.xiaonuo.biz.modular.depotWorkAlarmSet.param.DepotWorkAlarmSetAddParam;
import vip.xiaonuo.biz.modular.depotWorkAlarmSet.param.DepotWorkAlarmSetEditParam;
import vip.xiaonuo.biz.modular.depotWorkAlarmSet.param.DepotWorkAlarmSetIdParam;
import vip.xiaonuo.biz.modular.depotWorkAlarmSet.param.DepotWorkAlarmSetPageParam;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 出入库异常配置表Service接口实现类
 *
 * @author xiang
 * @date 2023/12/06 09:02
 **/
@Service
public class DepotWorkAlarmSetService extends ServiceImpl<DepotWorkAlarmSetMapper, DepotWorkAlarmSet> {

    public Page<DepotWorkAlarmSet> page(DepotWorkAlarmSetPageParam depotWorkAlarmSetPageParam) {
        QueryWrapper<DepotWorkAlarmSet> queryWrapper = new QueryWrapper<>();
        if (ObjectUtil.isNotEmpty(depotWorkAlarmSetPageParam.getAlarmType())) {
            queryWrapper.lambda().eq(DepotWorkAlarmSet::getAlarmType, depotWorkAlarmSetPageParam.getAlarmType());
        }
        if (ObjectUtil.isNotEmpty(depotWorkAlarmSetPageParam.getAlarmTitle())) {
            queryWrapper.lambda().like(DepotWorkAlarmSet::getAlarmTitle, depotWorkAlarmSetPageParam.getAlarmTitle());
        }
        if (ObjectUtil.isAllNotEmpty(depotWorkAlarmSetPageParam.getSortField(), depotWorkAlarmSetPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(depotWorkAlarmSetPageParam.getSortOrder());
            queryWrapper.orderBy(true, depotWorkAlarmSetPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(depotWorkAlarmSetPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(DepotWorkAlarmSet::getFlowType);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    public List<DepotWorkAlarmSet> list(String flowType) {
        LambdaQueryWrapper<DepotWorkAlarmSet> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Strings.isNotBlank(flowType), DepotWorkAlarmSet::getFlowType, flowType);
        return list(wrapper);
    }

    /**
     * 判断类型报警
     *
     * @param flowType 分类
     * @author xiang
     * @date 2023/12/06 09:02
     **/
    public boolean forBaoJing(String flowType) {
        QueryWrapper<DepotWorkAlarmSet> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(DepotWorkAlarmSet::getAlarmType, BizStr.ALARM_TYPE_B);
        wrapper.lambda().eq(DepotWorkAlarmSet::getFlowType, flowType);
        return count(wrapper) > 0;
    }

    public void add(DepotWorkAlarmSetAddParam depotWorkAlarmSetAddParam) {
        checkRepeatFlowType(null, depotWorkAlarmSetAddParam.getFlowType());
        DepotWorkAlarmSet depotWorkAlarmSet = BeanUtil.toBean(depotWorkAlarmSetAddParam, DepotWorkAlarmSet.class);
        this.save(depotWorkAlarmSet);
    }

    public void edit(DepotWorkAlarmSetEditParam depotWorkAlarmSetEditParam) {
        checkRepeatFlowType(depotWorkAlarmSetEditParam.getId(), depotWorkAlarmSetEditParam.getFlowType());
        DepotWorkAlarmSet depotWorkAlarmSet = this.queryEntity(depotWorkAlarmSetEditParam.getId());
        BeanUtil.copyProperties(depotWorkAlarmSetEditParam, depotWorkAlarmSet);
        this.updateById(depotWorkAlarmSet);
    }

    /**
     * 检查名称重复
     *
     * @author xiang
     * @date 2023-01-01
     */
    private void checkRepeatFlowType(String id, String flowType) {
        if (flowType == null) return;
        List<DepotWorkAlarmSet> list = list(flowType);
        if (id != null)
            list = list(flowType).stream().filter(one -> !one.getId().equals(id)).collect(Collectors.toList());
        if (list.isEmpty()) return;
        throw new CommonException(BizExceptionEnum.ERROR00010.getMessage());
    }

    @Transactional(rollbackFor = Exception.class)
    public void delete(List<DepotWorkAlarmSetIdParam> depotWorkAlarmSetIdParamList) {
        // 执行删除
        this.removeBatchByIds(CollStreamUtil.toList(depotWorkAlarmSetIdParamList, DepotWorkAlarmSetIdParam::getId));
    }

    public DepotWorkAlarmSet detail(DepotWorkAlarmSetIdParam depotWorkAlarmSetIdParam) {
        return this.queryEntity(depotWorkAlarmSetIdParam.getId());
    }

    public DepotWorkAlarmSet queryEntity(String id) {
        DepotWorkAlarmSet depotWorkAlarmSet = this.getById(id);
        if (ObjectUtil.isEmpty(depotWorkAlarmSet)) {
            throw new CommonException("出入库异常配置表不存在，id值为：{}", id);
        }
        return depotWorkAlarmSet;
    }

}
