/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotZoneFutures.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotZoneFutures.entity.DepotZoneFutures;
import vip.xiaonuo.biz.modular.depotZoneFutures.param.DepotZoneFuturesAddParam;
import vip.xiaonuo.biz.modular.depotZoneFutures.param.DepotZoneFuturesEditParam;
import vip.xiaonuo.biz.modular.depotZoneFutures.param.DepotZoneFuturesIdParam;
import vip.xiaonuo.biz.modular.depotZoneFutures.param.DepotZoneFuturesPageParam;
import vip.xiaonuo.biz.modular.depotZoneFutures.service.DepotZoneFuturesService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 期货信息控制器
 *
 * @author xiang
 * @date 2024/01/10 09:54
 */
@Api(tags = "期货信息控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotZoneFuturesController {

    @Resource
    private DepotZoneFuturesService depotZoneFuturesService;

    /**
     * 获取期货信息分页
     *
     * @author xiang
     * @date 2024/01/10 09:54
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取期货信息分页")
    @GetMapping("/biz/depotZoneFutures/page")
    public CommonResult<Page<DepotZoneFutures>> page(DepotZoneFuturesPageParam depotZoneFuturesPageParam) {
        return CommonResult.data(depotZoneFuturesService.page(depotZoneFuturesPageParam));
    }

    /**
     * 添加期货信息
     *
     * @author xiang
     * @date 2024/01/10 09:54
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加期货信息")
    @CommonLog("添加期货信息")
    @SaCheckPermission("/biz/depotZoneFutures/add")
    @PostMapping("/biz/depotZoneFutures/add")
    public CommonResult<String> add(@RequestBody @Valid DepotZoneFuturesAddParam depotZoneFuturesAddParam) {
        depotZoneFuturesService.add(depotZoneFuturesAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑期货信息
     *
     * @author xiang
     * @date 2024/01/10 09:54
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑期货信息")
    @CommonLog("编辑期货信息")
    @SaCheckPermission("/biz/depotZoneFutures/edit")
    @PostMapping("/biz/depotZoneFutures/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotZoneFuturesEditParam depotZoneFuturesEditParam) {
        depotZoneFuturesService.edit(depotZoneFuturesEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除期货信息
     *
     * @author xiang
     * @date 2024/01/10 09:54
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除期货信息")
    @CommonLog("删除期货信息")
    @SaCheckPermission("/biz/depotZoneFutures/delete")
    @PostMapping("/biz/depotZoneFutures/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotZoneFuturesIdParam> depotZoneFuturesIdParamList) {
        depotZoneFuturesService.delete(depotZoneFuturesIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取期货信息详情
     *
     * @author xiang
     * @date 2024/01/10 09:54
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取期货信息详情")
    @GetMapping("/biz/depotZoneFutures/detail")
    public CommonResult<DepotZoneFutures> detail(@Valid DepotZoneFuturesIdParam depotZoneFuturesIdParam) {
        return CommonResult.data(depotZoneFuturesService.detail(depotZoneFuturesIdParam));
    }
}
