/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotZonePrice.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotZonePrice.entity.DepotZonePrice;
import vip.xiaonuo.biz.modular.depotZonePrice.param.DepotZonePriceAddParam;
import vip.xiaonuo.biz.modular.depotZonePrice.param.DepotZonePriceEditParam;
import vip.xiaonuo.biz.modular.depotZonePrice.param.DepotZonePriceIdParam;
import vip.xiaonuo.biz.modular.depotZonePrice.param.DepotZonePricePageParam;
import vip.xiaonuo.biz.modular.depotZonePrice.service.DepotZonePriceService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 区域价格控制器
 *
 * @author xiang
 * @date  2023/09/18 17:38
 */
@Api(tags = "区域价格控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotZonePriceController {

    @Resource
    private DepotZonePriceService depotZonePriceService;

    /**
     * 获取区域价格分页
     *
     * @author xiang
     * @date  2023/09/18 17:38
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取区域价格分页")
    @SaCheckPermission("/biz/depotZonePrice/page")
    @GetMapping("/biz/depotZonePrice/page")
    public CommonResult<Page<DepotZonePrice>> page(DepotZonePricePageParam depotZonePricePageParam) {
        return CommonResult.data(depotZonePriceService.page(depotZonePricePageParam));
    }

    /**
     * 添加区域价格
     *
     * @author xiang
     * @date  2023/09/18 17:38
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加区域价格")
    @CommonLog("添加区域价格")
    @SaCheckPermission("/biz/depotZonePrice/add")
    @PostMapping("/biz/depotZonePrice/add")
    public CommonResult<String> add(@RequestBody @Valid DepotZonePriceAddParam depotZonePriceAddParam) {
        depotZonePriceService.add(depotZonePriceAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑区域价格
     *
     * @author xiang
     * @date  2023/09/18 17:38
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑区域价格")
    @CommonLog("编辑区域价格")
    @SaCheckPermission("/biz/depotZonePrice/edit")
    @PostMapping("/biz/depotZonePrice/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotZonePriceEditParam depotZonePriceEditParam) {
        depotZonePriceService.edit(depotZonePriceEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除区域价格
     *
     * @author xiang
     * @date  2023/09/18 17:38
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除区域价格")
    @CommonLog("删除区域价格")
    @SaCheckPermission("/biz/depotZonePrice/delete")
    @PostMapping("/biz/depotZonePrice/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<DepotZonePriceIdParam> depotZonePriceIdParamList) {
        depotZonePriceService.delete(depotZonePriceIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取区域价格详情
     *
     * @author xiang
     * @date  2023/09/18 17:38
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取区域价格详情")
    @SaCheckPermission("/biz/depotZonePrice/detail")
    @GetMapping("/biz/depotZonePrice/detail")
    public CommonResult<DepotZonePrice> detail(@Valid DepotZonePriceIdParam depotZonePriceIdParam) {
        return CommonResult.data(depotZonePriceService.detail(depotZonePriceIdParam));
    }
}
