/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.sysUserWxOpenSendSet.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.sysUserWxOpenSendSet.entity.SysUserWxOpenSendSet;
import vip.xiaonuo.biz.modular.sysUserWxOpenSendSet.param.SysUserWxOpenSendSetAddParam;
import vip.xiaonuo.biz.modular.sysUserWxOpenSendSet.param.SysUserWxOpenSendSetEditParam;
import vip.xiaonuo.biz.modular.sysUserWxOpenSendSet.param.SysUserWxOpenSendSetIdParam;
import vip.xiaonuo.biz.modular.sysUserWxOpenSendSet.param.SysUserWxOpenSendSetPageParam;
import vip.xiaonuo.biz.modular.sysUserWxOpenSendSet.service.SysUserWxOpenSendSetService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 用户微信推送内容订阅表控制器
 *
 * @author xiang
 * @date 2023/12/13 09:05
 */
@Api(tags = "用户微信推送内容订阅表控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class SysUserWxOpenSendSetController {

    @Resource
    private SysUserWxOpenSendSetService sysUserWxOpenSendSetService;

    /**
     * 获取用户微信推送内容订阅表分页
     *
     * @author xiang
     * @date 2023/12/13 09:05
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取用户微信推送内容订阅表分页")
    @SaCheckPermission("/biz/sysUserWxOpenSendSet/page")
    @GetMapping("/biz/sysUserWxOpenSendSet/page")
    public CommonResult<Page<SysUserWxOpenSendSet>> page(SysUserWxOpenSendSetPageParam sysUserWxOpenSendSetPageParam) {
        return CommonResult.data(sysUserWxOpenSendSetService.page(sysUserWxOpenSendSetPageParam));
    }

    /**
     * 添加用户微信推送内容订阅表
     *
     * @author xiang
     * @date 2023/12/13 09:05
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加用户微信推送内容订阅表")
    @CommonLog("添加用户微信推送内容订阅表")
    @SaCheckPermission("/biz/sysUserWxOpenSendSet/add")
    @PostMapping("/biz/sysUserWxOpenSendSet/add")
    public CommonResult<String> add(@RequestBody @Valid SysUserWxOpenSendSetAddParam sysUserWxOpenSendSetAddParam) {
        sysUserWxOpenSendSetService.add(sysUserWxOpenSendSetAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑用户微信推送内容订阅表
     *
     * @author xiang
     * @date 2023/12/13 09:05
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑用户微信推送内容订阅表")
    @CommonLog("编辑用户微信推送内容订阅表")
    @SaCheckPermission("/biz/sysUserWxOpenSendSet/edit")
    @PostMapping("/biz/sysUserWxOpenSendSet/edit")
    public CommonResult<String> edit(@RequestBody @Valid SysUserWxOpenSendSetEditParam sysUserWxOpenSendSetEditParam) {
        sysUserWxOpenSendSetService.edit(sysUserWxOpenSendSetEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除用户微信推送内容订阅表
     *
     * @author xiang
     * @date 2023/12/13 09:05
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除用户微信推送内容订阅表")
    @CommonLog("删除用户微信推送内容订阅表")
    @SaCheckPermission("/biz/sysUserWxOpenSendSet/delete")
    @PostMapping("/biz/sysUserWxOpenSendSet/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<SysUserWxOpenSendSetIdParam> sysUserWxOpenSendSetIdParamList) {
        sysUserWxOpenSendSetService.delete(sysUserWxOpenSendSetIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取用户微信推送内容订阅表详情
     *
     * @author xiang
     * @date 2023/12/13 09:05
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取用户微信推送内容订阅表详情")
    @SaCheckPermission("/biz/sysUserWxOpenSendSet/detail")
    @GetMapping("/biz/sysUserWxOpenSendSet/detail")
    public CommonResult<SysUserWxOpenSendSet> detail(@Valid SysUserWxOpenSendSetIdParam sysUserWxOpenSendSetIdParam) {
        return CommonResult.data(sysUserWxOpenSendSetService.detail(sysUserWxOpenSendSetIdParam));
    }
}
